
#include "main.h"
#include "args_edit.h"

ArgValueDialog::ArgValueDialog(BYTE value, argtype_t *arg_type)
:	wxDialog(NULL, -1, "Change Arg Value", wxDefaultPosition, wxDefaultSize)
{
	this->arg_type = arg_type;
	this->arg = value;

	wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
	SetSizer(vbox);

	// Values
	if (arg_type->has_values)
	{
		wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
		vbox->Add(hbox, 0, wxEXPAND|wxALL, 4);

		hbox->Add(new wxStaticText(this, -1, "Value:"), 1, wxALIGN_CENTER_VERTICAL);

		wxArrayString values;
		int index = 0;
		for (int a = 0; a < arg_type->values.size(); a++)
		{
			if (!arg_type->values[a].flag)
			{
				values.Add(arg_type->values[a].name.c_str());
				this->values.push_back(arg_type->values[a].value);
				if (arg_type->values[a].value == value)
					index = a;
			}
		}

		combo_value = new wxChoice(this, -1, wxDefaultPosition, wxDefaultSize, values);
		combo_value->Select(index);
		hbox->Add(combo_value, 0, wxEXPAND|wxLEFT, 4);
	}

	// Flags
	if (arg_type->has_flags)
	{
		wxStaticBox *frame = new wxStaticBox(this, -1, "Flags");
		wxStaticBoxSizer *box = new wxStaticBoxSizer(frame, wxVERTICAL);
		vbox->Add(box, 0, wxEXPAND|wxALL, 4);

		for (int a = 0; a < arg_type->values.size(); a++)
		{
			if (arg_type->values[a].flag)
			{
				flags.push_back(arg_type->values[a].value);
				wxCheckBox *cbox = new wxCheckBox(this, -1, arg_type->values[a].name.c_str());
				cbox_flags.push_back(cbox);
				box->Add(cbox, 0, wxEXPAND|wxBOTTOM, 2);

				if (arg_type->values[a].value & value)
					cbox->SetValue(true);
			}
		}
	}

	vbox->Add(CreateButtonSizer(wxOK|wxCANCEL), 0, wxEXPAND|wxALL, 4);

	Layout();
	SetBestFittingSize();
}

ArgValueDialog::~ArgValueDialog()
{
}

BYTE ArgValueDialog::get_value()
{
	BYTE val = 0;
	
	if (combo_value)
		val = values[combo_value->GetSelection()];

	for (int a = 0; a < cbox_flags.size(); a++)
	{
		if (cbox_flags[a]->GetValue())
			val |= flags[a];
	}

	return val;
}

ArgEdit::ArgEdit(wxWindow *parent, int val, string argname, argtype_t *arg_type)
:	wxPanel(parent, -1)
{
	this->argname = argname;
	this->arg_type = arg_type;

	wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
	SetSizer(hbox);

	label_arg = new wxStaticText(this, -1, wxString::Format("%s", argname.c_str()));
	hbox->Add(label_arg, 1, wxALIGN_CENTER_VERTICAL);

	if (arg_type && val != -1)
		label_arg->SetLabel(wxString::Format("%s: %s", argname.c_str(), arg_type->get_name(val).c_str()));

	label_arg->Wrap(200);

	entry_arg = new wxTextCtrl(this, AE_ENTRY_ARG, "", wxDefaultPosition, wxSize(32, -1));
	hbox->Add(entry_arg, 0, wxRIGHT, 4);

	if (val != -1)
		entry_arg->SetValue(wxString::Format("%d", val));

	btn_change = new wxButton(this, AE_BTN_CHANGE, "Change");
	hbox->Add(btn_change, 0);
}

ArgEdit::~ArgEdit()
{
}

BEGIN_EVENT_TABLE(ArgEdit, wxPanel)
	EVT_TEXT(AE_ENTRY_ARG, ArgEdit::entry_arg_changed)
	EVT_BUTTON(AE_BTN_CHANGE, ArgEdit::change_clicked)
END_EVENT_TABLE()

void ArgEdit::entry_arg_changed(wxCommandEvent &event)
{
	if (entry_arg->GetValue() == "")
		label_arg->SetLabel(wxString::Format("%s", argname.c_str()));
	else if (arg_type)
	{
		int val = atoi(entry_arg->GetValue());
		label_arg->SetLabel(wxString::Format("%s: %s", argname.c_str(), arg_type->get_name(val).c_str()));
		label_arg->Wrap(200);

		GetParent()->SetBestFittingSize();
		GetParent()->SetSize(360, -1);
		GetParent()->Refresh();
	}
}

void ArgEdit::change_clicked(wxCommandEvent &event)
{
	if (!arg_type)
		return;

	int val = 0;
	if (entry_arg->GetValue() != "")
		val = atoi(entry_arg->GetValue());

	ArgValueDialog av(val, arg_type);
	if (av.ShowModal() == wxID_OK)
		entry_arg->SetValue(wxString::Format("%d", av.get_value()));
}

int ArgEdit::get_value()
{
	int ret = -1;

	if (entry_arg->GetValue() != "")
	{
		ret = atoi(entry_arg->GetValue());

		if (ret > 255)
			ret = 255;
		if (ret < 0)
			ret = 0;
	}

	return ret;
}

ArgsEditor::ArgsEditor(BYTE *args, string *argnames, string *argtypes, bool *consistent)
:	wxDialog(NULL, -1, "Edit Args", wxDefaultPosition, wxDefaultSize)
{
	wxBoxSizer *m_vbox = new wxBoxSizer(wxVERTICAL);
	SetSizer(m_vbox);

	for (int a = 0; a < 5; a++)
	{
		int val = -1;
		if (consistent[a]) val = args[a];

		this->args[a] = new ArgEdit(this, val, argnames[a], get_arg_type(argtypes[a]));
		m_vbox->Add(this->args[a], 0, wxEXPAND|wxALL, 4);
	}

	m_vbox->Add(CreateButtonSizer(wxOK|wxCANCEL), 0, wxEXPAND|wxALL, 4);

	SetBestFittingSize();
	SetSize(360, -1);
	Layout();
}

ArgsEditor::~ArgsEditor()
{
}

void ArgsEditor::apply_changes(BYTE* args, bool *consistent)
{
	for (int a = 0; a < 5; a++)
	{
		if (this->args[a]->get_value() == -1)
			consistent[a] = false;
		else
		{
			consistent[a] = true;
			args[a] = this->args[a]->get_value();
		}
	}
}
