// << ------------------------------------- >>
// << SLADE - SlayeR's LeetAss Doom Editor  >>
// << By Simon Judd, 2004-05                >>
// << ------------------------------------- >>
// << edit_misc.cpp - Various editing tools >>
// << (for the 'edit' menu)                 >>
// << ------------------------------------- >>

// INCLUDES ------------------------------ >>
#include "main.h"
#include "map.h"
#include "undoredo.h"
#include "struct_3d.h"
#include "mathstuff.h"
#include "textures.h"
#include "info_bar.h"
#include "editor_window.h"

// Variables ----------------------------- >>
string def_doortex = "";
string def_tracktex = "DOORTRAK";

// External Variables -------------------- >>
extern Map map;
extern vector<int> selected_items;
extern int edit_mode;
extern int hilight_item;
extern EditorWindow *editor_window;

// SECTORS

// sector_create_stairs: Creates stairs from a group of sectors
// --------------------------------------------------------- >>
void sector_create_stairs(int floor_step, int ceil_step)
{
	if (selected_items.size() <= 1 || edit_mode != 2)
		return;

	make_backup(false, false, false, true, false);

	// Floor
	int height = map.sectors[selected_items[0]]->f_height;
	for (DWORD a = 0; a < selected_items.size(); a++)
	{
		height += floor_step;
		map.sectors[selected_items[a]]->f_height = height;
	}

	// Ceiling
	height = map.sectors[selected_items[0]]->c_height;
	for (DWORD a = 0; a < selected_items.size(); a++)
	{
		height += ceil_step;
		map.sectors[selected_items[a]]->c_height = height;
	}

	//map_changelevel(2);
	map.change_level(MC_SSECTS|MC_LINES);
}

// sector_light_gradient: Creates a light gradient
// -------------------------------------------- >>
void sector_light_gradient(int step)
{
	if (selected_items.size() <= 1 || edit_mode != 2)
		return;

	make_backup(false, false, false, true, false);

	if (step > 255)
	{
		int start = map.sectors[selected_items[0]]->light;
		int end = map.sectors[selected_items.back()]->light;
		step = float(end - start) / selected_items.size();
	}

	int l = map.sectors[selected_items[0]]->light;
	for (int a = 0; a < selected_items.size(); a++)
	{
		if (l > 255)
			l = 255;
		if (l < 0)
			l = 0;

		map.sectors[selected_items[a]]->light = l;

		l += step;
	}

	//map_changelevel(2);
	map.change_level(MC_SECTORS);
}

// sector_create_door: Creates a door from any selected sectors
// --------------------------------------------------------- >>
void sector_create_door(string texture)
{
	if ((selected_items.size() == 0 && hilight_item == -1) || edit_mode != 2)
		return;

	make_backup(true, true, false, true, false);

	vector<int> items;

	if (selected_items.size() == 0)
		items.push_back(hilight_item);
	else
	{
		for (int a = 0; a < selected_items.size(); a++)
			items.push_back(selected_items[a]);
	}

	for (DWORD a = 0; a < items.size(); a++)
	{
		int s = items[a];

		// Set ceiling to floor
		map.sectors[s]->c_height = map.sectors[s]->f_height;
		
		// Go through lines in the sector, set their texture and type accordingly
		for (DWORD l = 0; l < map.n_lines; l++)
		{
			if (map.lines[l]->side2 != -1)
			{
				// Any 2-sided lines
				if (map.sides[map.lines[l]->side2]->sector == s)
				{
					map.sides[map.lines[l]->side1]->tex_upper = texture;
					
					if (!map.zdoom)
						map.lines[l]->type = 1;
					else
					{
						map.lines[l]->type = 12;
						map.lines[l]->args[1] = 24;
						map.lines[l]->args[2] = 128;
						map.lines[l]->set_flag(LINE_REPEATABLE);
						map.lines[l]->set_trigger(LINE_SPAC_USE);
					}
				}

				if (map.sides[map.lines[l]->side1]->sector == s)
				{
					map.sides[map.lines[l]->side2]->tex_upper = texture;

					if (!map.zdoom)
						map.lines[l]->type = 1;
					else
					{
						map.lines[l]->type = 12;
						map.lines[l]->args[1] = 24;
						map.lines[l]->args[2] = 128;
						map.lines[l]->set_flag(LINE_REPEATABLE);
						map.lines[l]->set_trigger(LINE_SPAC_USE);
					}
				}
			}
			else // 1-sided lines
			{
				if (map.sides[map.lines[l]->side1]->sector == s)
				{
					map.sides[map.lines[l]->side1]->tex_middle = def_tracktex;
					map.lines[l]->set_flag(LINE_LOWERUNPEGGED);
				}
			}
		}
	}

	//force_map_redraw(true, false);
	editor_window->redraw_map(true, false);
	map.change_level(MC_SSECTS|MC_LINES);
	//map_changelevel(2);
}

// sector_merge: Merges selected sectors to the first selected sector
// --------------------------------------------------------------- >>
void sector_merge(bool remove_lines)
{
	if (selected_items.size() == 0 || edit_mode != 2)
		return;

	make_backup(true, true, false, true, false);

	int sector = selected_items[0];

	for (int l = 0; l < map.n_lines; l++)
	{
		if (remove_lines)
		{
			// If both sides of the line are selected, delete the line
			if (vector_exists(selected_items, map.l_getsector1(l)) &&
				vector_exists(selected_items, map.l_getsector2(l)))
				map.delete_line(l);
		}

		if (vector_exists(selected_items, map.l_getsector1(l)))
			map.l_setsector(l, 1, sector);

		if (vector_exists(selected_items, map.l_getsector2(l)))
			map.l_setsector(l, 2, sector);
	}

	selected_items.clear();
	//force_map_redraw(true, false);
	editor_window->redraw_map(true);
	map.change_level(MC_NODE_REBUILD);
}

void sector_changelight(int amount)
{
	if (edit_mode != 2 || (selected_items.size() == 0 && hilight_item == -1))
		return;

	make_backup(false, false, false, true, false);

	vector<int> sectors;

	// Get sectors to change
	if (selected_items.size() == 0)
		sectors.push_back(hilight_item);
	else
	{
		for (int a = 0; a < selected_items.size(); a++)
			vector_add_nodup(sectors, selected_items[a]);
			//sectors.push_back(selected_items[a]);
	}

	// Change light levels
	for (int a = 0; a < sectors.size(); a++)
	{
		int light = map.sectors[sectors[a]]->light;

		if (light == 255)
			light = 256;

		light += amount;

		if (light < 0)
			light = 0;
		if (light > 255)
			light = 255;

		map.sectors[sectors[a]]->light = light;
	}

	//if (hilight_item != -1)
	//	update_sector_info_bar(hilight_item);
	if (hilight_item != -1)
		editor_window->update_infobar();
}

void sector_create(point2_t point)
{
	double min_dist = -1;
	int line = -1;

	for (WORD l = 0; l < map.n_lines; l++)
	{
		rect_t r = map.l_getrect(l);

		double dist = distance_to_line(r.x1(), r.y1(), r.x2(), r.y2(), point.x, point.y);

		if (min_dist == -1)
		{
			min_dist = dist;
			line = l;
		}
		else if (dist < min_dist)
		{
			min_dist = dist;
			line = l;
		}
	}

	if (line == -1)
		return;

	make_backup(true, true, false, true, false);

	vector<int> processed_lines;
	vector<int> second_sides;
	vector<int> first_sides;
	int vertex;

	if (determine_line_side(line, point.x, point.y))
		vertex = map.lines[line]->vertex2;
	else
		vertex = map.lines[line]->vertex1;

	sector_t* copy_sector = NULL;

	//wxLogMessage("\nCreate sector start line %d", line);

	bool done = false;
	while (!done)
	{
		point3_t vec1;
		processed_lines.push_back(line);

		if (vertex == map.lines[line]->vertex2)
		{
			first_sides.push_back(line);
			if (map.l_getside(line, 1) && !copy_sector)
				copy_sector = map.sectors[map.l_getside(line, 1)->sector];
			//vec1.set(map.l_getrect(line).br.x - map.l_getrect(line).tl.x, map.l_getrect(line).tl.y - map.l_getrect(line).br.y, 0.0f);
		}
		else
		{
			second_sides.push_back(line);
			if (map.l_getside(line, 2) && !copy_sector)
				copy_sector = map.sectors[map.l_getside(line, 2)->sector];
			//vec1.set(map.l_getrect(line).tl.x - map.l_getrect(line).br.x, map.l_getrect(line).br.y - map.l_getrect(line).tl.y, 0.0f);
		}

		point2_t v1 = map.v_getpoint(vertex);//map.l_getrect(a_lines.numbers[a]).tl;
		point2_t v2;// = map.l_getrect(a_lines.numbers[a]).br;

		if (vertex == map.lines[line]->vertex1)
			v2 = map.v_getpoint(map.lines[line]->vertex2);
		else
			v2 = map.v_getpoint(map.lines[line]->vertex1);

		vec1.set(v1.x - v2.x, v2.y - v1.y, 0.0f);
		vec1 = vec1.normalize();

		numlist_t a_lines;
		map.v_getattachedlines(vertex, &a_lines);
		//wxLogMessage("%d a_lines", a_lines.n_numbers);

		// Get line with lowest angle from the current line
		double lo_angle = 9999;
		int lo_line = -1;
		for (int a = 0; a < a_lines.n_numbers; a++)
		{
			//wxLogMessage("%d %d", line, a_lines.numbers[a]);

			if (a_lines.numbers[a] == line)
				continue;

			if (vector_exists(processed_lines, a_lines.numbers[a]))
			{
				done = true;
				break;
			}

			point3_t vec2;
			v1 = map.v_getpoint(vertex);

			if (vertex == map.lines[a_lines.numbers[a]]->vertex1)
				v2 = map.v_getpoint(map.lines[a_lines.numbers[a]]->vertex2);
			else
				v2 = map.v_getpoint(map.lines[a_lines.numbers[a]]->vertex1);

			vec2.set(v1.x - v2.x, v2.y - v1.y, 0.0f);
			vec2 = vec2.normalize();

			double angle = get_2d_angle(vec1, vec2);
			//wxLogMessage("line %d: %1.2f,%1.2f   %1.2f,%1.2f", a_lines.numbers[a], vec1.x, vec1.y, vec2.x, vec2.y);
			//wxLogMessage("angle between line %d and %d: %1.3f", line, a_lines.numbers[a], angle);

			if (angle < lo_angle)
			{
				lo_angle = angle;
				lo_line = a_lines.numbers[a];
			}
		}

		if (lo_line != -1)
		{
			line = lo_line;

			if (vertex == map.lines[line]->vertex1)
				vertex = map.lines[line]->vertex2;
			else
				vertex = map.lines[line]->vertex1;
		}
		else
			done = true;
	}

	int sector = map.add_sector();

	if (copy_sector)
		memcpy(map.sectors[sector], copy_sector, sizeof(sector_t));

	for (int a = 0; a < first_sides.size(); a++)
		map.l_setsector(first_sides[a], 1, sector);

	for (int a = 0; a < second_sides.size(); a++)
		map.l_setsector(second_sides[a], 2, sector);

	map.change_level(MC_NODE_REBUILD|MC_SECTORS|MC_LINES);
	editor_window->redraw_map(true, false);
}

void sector_paint_tex(int sector, string tex, bool floor, vector<int> *sectors)
{
	if (sector < 0 || sector > map.n_sectors)
		return;

	sectors->push_back(sector);
	for (int a = 0; a < map.n_lines; a++)
	{
		int s1 = map.l_getsector1(a);
		int s2 = map.l_getsector2(a);

		if (vector_exists((*sectors), s1) && s2 != -1 && !(vector_exists((*sectors), s2)))
		{
			bool match = false;

			if (floor && map.sectors[s1]->f_height == map.sectors[s2]->f_height)
				match = true;

			if (!floor && map.sectors[s1]->c_height == map.sectors[s2]->c_height)
				match = true;

			if (match)
			{
				vector_add_nodup((*sectors), s2);
				a = -1;
				continue;
			}
		}

		if (vector_exists((*sectors), s2) && s1 != -1 && !(vector_exists((*sectors), s1)))
		{
			bool match = false;

			if (floor && map.sectors[s1]->f_height == map.sectors[s2]->f_height)
				match = true;

			if (!floor && map.sectors[s1]->c_height == map.sectors[s2]->c_height)
				match = true;

			if (match)
			{
				vector_add_nodup((*sectors), s1);
				a = -1;
				continue;
			}
		}
	}

	for (int a = 0; a < sectors->size(); a++)
	{
		int s = (*sectors)[a];

		if (floor)
			map.sectors[s]->f_tex = tex;
		else
			map.sectors[s]->c_tex = tex;
	}
}

// LINES

// line_extrude: 'extrudes' selected lines by a specified amount
// ---------------------------------------------------------- >>
void line_extrude(int amount)
{
	if ((selected_items.size() == 0 && hilight_item == -1) || edit_mode != 1)
		return;

	make_backup(true, true, true, false, false);

	if (selected_items.size() > 0)
	{
		for (int a = 0; a < selected_items.size(); a++)
		{
			int l = selected_items[a];

			// Get a perpendicular vector to the line
			point2_t line(map.l_getrect(l).x2() - map.l_getrect(l).x1(), map.l_getrect(l).y2() - map.l_getrect(l).y1());
			point3_t vec(-line.y, line.x, 0.0f);
			vec = vec.normalize();

			// Add the new vertices
			int v1 = map.add_vertex(map.l_getrect(l).x1() - (vec.x * amount), map.l_getrect(l).y1() - (vec.y * amount));
			int v2 = map.add_vertex(map.l_getrect(l).x2() - (vec.x * amount), map.l_getrect(l).y2() - (vec.y * amount));

			// Split the line through the new vertices
			map.l_split(l, v2);
			map.l_split(l, v1);
		}
	}
	else
	{
		int l = hilight_item;

		// Get a perpendicular vector to the line
		point2_t line(map.l_getrect(l).x2() - map.l_getrect(l).x1(), map.l_getrect(l).y2() - map.l_getrect(l).y1());
		point3_t vec(-line.y, line.x, 0.0f);
		vec = vec.normalize();

		// Add the new vertices
		int v1 = map.add_vertex(map.l_getrect(l).x1() - (vec.x * amount), map.l_getrect(l).y1() - (vec.y * amount));
		int v2 = map.add_vertex(map.l_getrect(l).x2() - (vec.x * amount), map.l_getrect(l).y2() - (vec.y * amount));

		// Split the line through the new vertices
		map.l_split(l, v2);
		map.l_split(l, v1);
	}

	selected_items.clear();
	map.change_level(MC_NODE_REBUILD);
	//force_map_redraw(true, false);
	editor_window->redraw_map(true);
}

void line_align_x()
{
	if (selected_items.size() <= 1 || edit_mode != 1)
		return;

	int offset = map.l_getrect(selected_items[0]).length();
	for (int a = 1; a < selected_items.size(); a++)
	{
		sidedef_t *side = map.l_getside(selected_items[a], 1);
		if (side) side->x_offset = offset;

		side = map.l_getside(selected_items[a], 2);
		if (side) side->x_offset = offset;

		offset += map.l_getrect(selected_items[a]).length();
	}
}

void line_align_y()
{
}

void line_correct_references(int line)
{
	if (line != -1)
	{
		map.l_setsector(line, 1, get_side_sector(line, 1));
		map.l_setsector(line, 2, get_side_sector(line, 2));

		return;
	}

	if (edit_mode != 1)
		return;

	if (selected_items.size() == 0 && hilight_item != -1)
	{
		map.l_setsector(hilight_item, 1, get_side_sector(hilight_item, 1));
		map.l_setsector(hilight_item, 2, get_side_sector(hilight_item, 2));
	}
	else if (selected_items.size() > 0)
	{
		for (int a = 0; a < selected_items.size(); a++)
		{
			map.l_setsector(selected_items[a], 1, get_side_sector(selected_items[a], 1));
			map.l_setsector(selected_items[a], 2, get_side_sector(selected_items[a], 2));
		}
	}
}

void line_auto_align_x(int line, int offset, int side, string tex, int texwidth, vector<int> *processed_lines)
{
	if (offset == -51403)
		offset = map.l_getxoff(line, side);
	else
		map.l_getside(line, side)->x_offset = offset;

	offset += map.l_getrect(line).length();
	processed_lines->push_back(line);

	if (offset >= texwidth)
		offset -= texwidth;

	numlist_t a_lines;

	if (side == 1)
		map.v_getattachedlines(map.lines[line]->vertex2, &a_lines);
	else
		map.v_getattachedlines(map.lines[line]->vertex1, &a_lines);

	for (int a = 0; a < a_lines.n_numbers; a++)
	{
		if (find(processed_lines->begin(), processed_lines->end(), a_lines.numbers[a]) == processed_lines->end())
		{
			if (map.l_getside(a_lines.numbers[a], 1))
			{
				if (map.l_getside(a_lines.numbers[a], 1)->tex_lower == tex ||
					map.l_getside(a_lines.numbers[a], 1)->tex_middle == tex ||
					map.l_getside(a_lines.numbers[a], 1)->tex_upper == tex)
					line_auto_align_x(a_lines.numbers[a], offset, 1, tex, texwidth, processed_lines);
			}

			if (map.l_getside(a_lines.numbers[a], 2))
			{
				if (map.l_getside(a_lines.numbers[a], 2)->tex_lower == tex ||
					map.l_getside(a_lines.numbers[a], 2)->tex_middle == tex ||
					map.l_getside(a_lines.numbers[a], 2)->tex_upper == tex)
					line_auto_align_x(a_lines.numbers[a], offset, 2, tex, texwidth, processed_lines);
			}
		}
	}
}

bool line_hastex(int line, int side, string tex)
{
	sidedef_t *sdef = map.l_getside(line, side);

	if (!sdef)
		return false;

	if (map.l_needsuptex(line, side) && sdef->tex_upper == tex)
		return true;
	if (map.l_needsmidtex(line, side) && sdef->tex_middle == tex)
		return true;
	if (map.l_needslotex(line, side) && sdef->tex_lower == tex)
		return true;

	return false;
}

void line_paint_tex(int line, int side, string otex, string ntex, vector<int> *processed_lines)
{
	sidedef_t *sdef = map.l_getside(line, side);

	if (sdef)
	{
		if (map.l_needslotex(line, side) && sdef->tex_lower == otex)
			sdef->tex_lower = ntex;
		if (map.l_needsmidtex(line, side) && sdef->tex_middle == otex)
			sdef->tex_middle = ntex;
		if (map.l_needsuptex(line, side) && sdef->tex_upper == otex)
			sdef->tex_upper = ntex;
	}

	processed_lines->push_back(line);

	numlist_t a_lines;

	map.v_getattachedlines(map.lines[line]->vertex2, &a_lines);
	map.v_getattachedlines(map.lines[line]->vertex1, &a_lines);

	for (int a = 0; a < a_lines.n_numbers; a++)
	{
		if (find(processed_lines->begin(), processed_lines->end(), a_lines.numbers[a]) == processed_lines->end())
		{
			if (line_hastex(a_lines.numbers[a], 1, otex))
				line_paint_tex(a_lines.numbers[a], 1, otex, ntex, processed_lines);

			if (line_hastex(a_lines.numbers[a], 2, otex))
				line_paint_tex(a_lines.numbers[a], 2, otex, ntex, processed_lines);
		}
	}
}

void line_split(int splits)
{
	if (selected_items.size() == 0 && hilight_item == -1)
		return;

	make_backup(true, true, true, false, false);

	if (selected_items.size() == 0)
	{
		rect_t rect = map.l_getrect(hilight_item);

		float x_split = float(rect.x2() - rect.x1()) / (float)splits;
		float y_split = float(rect.y2() - rect.y1()) / (float)splits;

		int line = hilight_item;
		for (int a = 1; a < splits; a++)
		{
			int x = rect.x1() + int(a * x_split);
			int y = rect.y1() + int(a * y_split);

			int v = map.add_vertex(x, y);
			line = map.l_split(line, v);
		}
	}
	else
	{
		for (int a = 0; a < selected_items.size(); a++)
		{
			rect_t rect = map.l_getrect(selected_items[a]);

			float x_split = float(rect.x2() - rect.x1()) / (float)splits;
			float y_split = float(rect.y2() - rect.y1()) / (float)splits;

			int line = selected_items[a];
			for (int b = 1; b < splits; b++)
			{
				int x = rect.x1() + int(b * x_split);
				int y = rect.y1() + int(b * y_split);

				int v = map.add_vertex(x, y);
				line = map.l_split(line, v);
			}
		}
	}

	editor_window->redraw_map(true, false);
	//force_map_redraw(true, false);
}


// VERTICES
void get_selected_verts(vector<vertex_t*> *items, vector<int> *lines)
{
	
}

void vert_mirror(bool mirror_x, bool mirror_y)
{
	vector<vertex_t*> items;
	vector<int> lines;

	make_backup(true, false, true, false, false);

	//wxMessageBox("hgeh");
	// Gather vertices to mirror
	if (edit_mode == 0)
	{
		if (selected_items.size() > 0)
		{
			for (int a = 0; a < selected_items.size(); a++)
				items.push_back(map.verts[selected_items[a]]);
		}
		else
			return;
	}

	if (edit_mode == 1)
	{
		if (selected_items.size() > 0)
		{
			for (int a = 0; a < selected_items.size(); a++)
			{
				lines.push_back(selected_items[a]);
				vector_add_nodup(items, map.verts[map.lines[selected_items[a]]->vertex1]);
				vector_add_nodup(items, map.verts[map.lines[selected_items[a]]->vertex2]);
			}
		}
		else if (hilight_item != -1)
		{
			lines.push_back(hilight_item);
			vector_add_nodup(items, map.verts[map.lines[hilight_item]->vertex1]);
			vector_add_nodup(items, map.verts[map.lines[hilight_item]->vertex2]);
		}
	}

	if (edit_mode == 2)
	{
		for (int a = 0; a < map.n_lines; a++)
		{
			if (selected_items.size() > 0)
			{
				if (vector_exists(selected_items, map.l_getsector1(a))
					|| vector_exists(selected_items, map.l_getsector2(a)))
				{
					lines.push_back(a);
					vector_add_nodup(items, map.verts[map.lines[a]->vertex1]);
					vector_add_nodup(items, map.verts[map.lines[a]->vertex2]);
				}
			}
			else if (hilight_item != -1)
			{
				if (map.l_getsector1(a) == hilight_item || map.l_getsector2(a) == hilight_item)
				{
					lines.push_back(a);
					vector_add_nodup(items, map.verts[map.lines[a]->vertex1]);
					vector_add_nodup(items, map.verts[map.lines[a]->vertex2]);
				}
			}
		}
	}

	if (items.size() == 0)
		return;

	// Get midpoint
	fpoint2_t midpoint;

	int max_x = -32767;
	int max_y = -32767;
	int min_x = 32767;
	int min_y = 32767;
	for (int a = 0; a < items.size(); a++)
	{
		if (items[a]->x > max_x)
			max_x = items[a]->x;

		if (items[a]->x < min_x)
			min_x = items[a]->x;

		if (items[a]->y > max_y)
			max_y = items[a]->y;

		if (items[a]->y < min_y)
			min_y = items[a]->y;
	}

	midpoint.set(min_x + ((max_x - min_x) / 2.0),
				min_y + ((max_y - min_y) / 2.0));

	for (int a = 0; a < items.size(); a++)
	{
		if (mirror_x)
			items[a]->x = midpoint.x - (items[a]->x - midpoint.x);

		if (mirror_y)
			items[a]->y = midpoint.y - (items[a]->y - midpoint.y);
	}

	for (int a = 0; a < lines.size(); a++)
	{
		// Swap vertices (so lines don't go inside-out)
		int v1 = map.lines[lines[a]]->vertex1;
		map.lines[lines[a]]->vertex1 = map.lines[lines[a]]->vertex2;
		map.lines[lines[a]]->vertex2 = v1;
	}

	map.change_level(MC_NODE_REBUILD);
	editor_window->redraw_map(true, false);
}

void vert_rotate(double angle)
{
	vector<vertex_t*> items;
	//vector<int> lines;

	make_backup(false, false, true, false, false);

	// Gather vertices to rotate
	if (edit_mode == 0)
	{
		if (selected_items.size() > 0)
		{
			for (int a = 0; a < selected_items.size(); a++)
				items.push_back(map.verts[selected_items[a]]);
		}
		else
			return;
	}

	if (edit_mode == 1)
	{
		if (selected_items.size() > 0)
		{
			for (int a = 0; a < selected_items.size(); a++)
			{
				vector_add_nodup(items, map.verts[map.lines[selected_items[a]]->vertex1]);
				vector_add_nodup(items, map.verts[map.lines[selected_items[a]]->vertex2]);
			}
		}
		else if (hilight_item != -1)
		{
			vector_add_nodup(items, map.verts[map.lines[hilight_item]->vertex1]);
			vector_add_nodup(items, map.verts[map.lines[hilight_item]->vertex2]);
		}
	}

	if (edit_mode == 2)
	{
		for (int a = 0; a < map.n_lines; a++)
		{
			if (selected_items.size() > 0)
			{
				if (vector_exists(selected_items, map.l_getsector1(a))
					|| vector_exists(selected_items, map.l_getsector2(a)))
				{
					vector_add_nodup(items, map.verts[map.lines[a]->vertex1]);
					vector_add_nodup(items, map.verts[map.lines[a]->vertex2]);
				}
			}
			else if (hilight_item != -1)
			{
				if (map.l_getsector1(a) == hilight_item || map.l_getsector2(a) == hilight_item)
				{
					vector_add_nodup(items, map.verts[map.lines[a]->vertex1]);
					vector_add_nodup(items, map.verts[map.lines[a]->vertex2]);
				}
			}
		}
	}

	if (items.size() == 0)
		return;

	// Get midpoint
	double mid_x, mid_y;

	double max_x = -32767;
	double max_y = -32767;
	double min_x = 32767;
	double min_y = 32767;
	for (int a = 0; a < items.size(); a++)
	{
		if (items[a]->x > max_x)
			max_x = items[a]->x;

		if (items[a]->x < min_x)
			min_x = items[a]->x;

		if (items[a]->y > max_y)
			max_y = items[a]->y;

		if (items[a]->y < min_y)
			min_y = items[a]->y;
	}

	mid_x = min_x + ((max_x - min_x) * 0.5);
	mid_y = min_y + ((max_y - min_y) * 0.5);

	double rot = (3.1415926535897932384626433832795 * 2) * ((360.0 - angle) / 360.0);
	double srot = sin(rot);
	double crot = cos(rot);

	for (int a = 0; a < items.size(); a++)
	{
		double x = items[a]->x - round(mid_x);
		double y = items[a]->y - round(mid_y);

		double nx = crot * x - srot * y;
		double ny = srot * x + crot * y;

		items[a]->x = mid_x + round(nx);
		items[a]->y = mid_y + round(ny);
	}

	map.change_level(MC_NODE_REBUILD);
	editor_window->redraw_map(true, false);
}


// THINGS
void thing_mirror(bool mirror_x, bool mirror_y, bool angles)
{
	vector<thing_t*> items;

	make_backup(false, false, false, false, true);

	// Gather things to mirror

	if (edit_mode == 3)
	{
		if (selected_items.size() > 0)
		{
			for (int a = 0; a < selected_items.size(); a++)
				items.push_back(map.things[selected_items[a]]);
		}
		else
			return;
	}

	if (items.size() == 0)
		return;

	// Get midpoint
	fpoint2_t midpoint;

	int max_x = -32767;
	int max_y = -32767;
	int min_x = 32767;
	int min_y = 32767;
	for (int a = 0; a < items.size(); a++)
	{
		if (items[a]->x > max_x)
			max_x = items[a]->x;

		if (items[a]->x < min_x)
			min_x = items[a]->x;

		if (items[a]->y > max_y)
			max_y = items[a]->y;

		if (items[a]->y < min_y)
			min_y = items[a]->y;
	}

	midpoint.set(min_x + ((max_x - min_x) / 2.0),
				min_y + ((max_y - min_y) / 2.0));

	for (int a = 0; a < items.size(); a++)
	{
		if (mirror_x)
		{
			items[a]->x = midpoint.x - (items[a]->x - midpoint.x);

			if (angles)
			{
				switch (items[a]->angle)
				{
				case 0:
					items[a]->angle = 180; break;
				case 45:
					items[a]->angle = 135; break;
				case 135:
					items[a]->angle = 45; break;
				case 180:
					items[a]->angle = 0; break;
				case 225:
					items[a]->angle = 315; break;
				case 315:
					items[a]->angle = 225; break;
				}
			}
		}

		if (mirror_y)
		{
			items[a]->y = midpoint.y - (items[a]->y - midpoint.y);

			if (angles)
			{
				switch (items[a]->angle)
				{
				case 90:
					items[a]->angle = 270; break;
				case 45:
					items[a]->angle = 315; break;
				case 315:
					items[a]->angle = 45; break;
				case 270:
					items[a]->angle = 90; break;
				case 135:
					items[a]->angle = 225; break;
				case 225:
					items[a]->angle = 135; break;
				}
			}
		}
	}

	map.change_level(MC_THINGS);
	editor_window->redraw_map(true, false);
}

void thing_rotate(double angle, bool rot_angles)
{
	vector<thing_t*> items;

	make_backup(false, false, false, false, true);

	// Gather things to mirror

	if (edit_mode == 3)
	{
		if (selected_items.size() > 0)
		{
			for (int a = 0; a < selected_items.size(); a++)
				items.push_back(map.things[selected_items[a]]);
		}
		else
			return;
	}

	if (items.size() == 0)
		return;

	// Get midpoint
	fpoint2_t midpoint;

	int max_x = -32767;
	int max_y = -32767;
	int min_x = 32767;
	int min_y = 32767;
	for (int a = 0; a < items.size(); a++)
	{
		if (items[a]->x > max_x)
			max_x = items[a]->x;

		if (items[a]->x < min_x)
			min_x = items[a]->x;

		if (items[a]->y > max_y)
			max_y = items[a]->y;

		if (items[a]->y < min_y)
			min_y = items[a]->y;
	}

	double mid_x = min_x + (double(max_x - min_x) * 0.5);
	double mid_y = min_y + (double(max_y - min_y) * 0.5);

	double rot = (3.1415926535897932384626433832795 * 2) * ((360.0 - angle) / 360.0);
	double srot = sin(rot);
	double crot = cos(rot);

	for (int a = 0; a < items.size(); a++)
	{
		double x = items[a]->x - round(mid_x);
		double y = items[a]->y - round(mid_y);

		double nx = crot * x - srot * y;
		double ny = srot * x + crot * y;

		items[a]->x = mid_x + round(nx);
		items[a]->y = mid_y + round(ny);

		if (rot_angles)
		{
			items[a]->angle += (angle - 90);

			if (items[a]->angle >= 360)
				items[a]->angle -= 360;
			if (items[a]->angle < 0)
				items[a]->angle += 360;

			if (items[a]->angle < 68 && items[a]->angle > 22)
				items[a]->angle = 45;

			if (items[a]->angle < 113 && items[a]->angle > 67)
				items[a]->angle = 90;

			if (items[a]->angle < 158 && items[a]->angle > 112)
				items[a]->angle = 135;

			if (items[a]->angle < 203 && items[a]->angle > 157)
				items[a]->angle = 180;

			if (items[a]->angle < 248 && items[a]->angle > 202)
				items[a]->angle = 225;

			if (items[a]->angle < 293 && items[a]->angle > 247)
				items[a]->angle = 270;

			if (items[a]->angle < 338 && items[a]->angle > 292)
				items[a]->angle = 315;

			if (items[a]->angle < 23 || items[a]->angle > 337)
				items[a]->angle = 0;
		}
	}
}
