
#include "main.h"
#include "info_bar.h"
#include "map.h"
#include "action_special.h"
#include "editor_window.h"
#include "args.h"
#include "struct_3d.h"
#include "mathstuff.h"

extern Map map;
extern int hilight_item;
extern EditorWindow *editor_window;

LineInfoBar::LineInfoBar(wxWindow *parent)
:	InfoBar(parent)
{
	wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
	SetSizer(hbox);

	// Main frame
	main_frame = new wxStaticBox(this, -1, "No Line Hilighted");
	wxFont def_font = main_frame->GetFont();
	def_font.SetWeight(wxFONTWEIGHT_BOLD);
	main_frame->SetFont(def_font);
	wxStaticBoxSizer *box = new wxStaticBoxSizer(main_frame, wxHORIZONTAL);
	hbox->Add(box, 1, wxEXPAND|wxRIGHT, 4);

	wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
	vbox->SetMinSize(160, -1);
	box->Add(vbox, 0, wxRIGHT, 8);

	label_length = new wxStaticText(this, -1, "Length:");
	label_front = new wxStaticText(this, -1, "Front Sector:");
	label_back = new wxStaticText(this, -1, "Back Sector:");
	label_special = new wxStaticText(this, -1, "Special:");
	label_tag = new wxStaticText(this, -1, "Sector Tag:");
	label_offsets1 = new wxStaticText(this, -1, "Front Offsets:");
	label_offsets2 = new wxStaticText(this, -1, "Back Offsets:");

	vbox->Add(label_length);
	vbox->Add(label_front);
	vbox->Add(label_back);
	vbox->Add(label_special);
	vbox->Add(label_tag);
	vbox->Add(label_offsets1);
	vbox->Add(label_offsets2);

	vbox = new wxBoxSizer(wxVERTICAL);
	box->Add(vbox);

	label_arg1 = new wxStaticText(this, -1, "");
	label_arg2 = new wxStaticText(this, -1, "");
	label_arg3 = new wxStaticText(this, -1, "");
	label_arg4 = new wxStaticText(this, -1, "");
	label_arg5 = new wxStaticText(this, -1, "");

	vbox->Add(label_arg1);
	vbox->Add(label_arg2);
	vbox->Add(label_arg3);
	vbox->Add(label_arg4);
	vbox->Add(label_arg5);

	// Front side
	front_frame = new wxStaticBox(this, -1, "Front Side");
	front_frame->SetFont(def_font);
	box = new wxStaticBoxSizer(front_frame, wxHORIZONTAL);
	box->SetMinSize(192, -1);
	hbox->Add(box, 0, wxEXPAND|wxRIGHT, 4);

	tex_upper1 = new TextureBox(this, 64, 64, "_none_", 1, 2.0f);
	tex_middle1 = new TextureBox(this, 64, 64, "_none_", 1, 2.0f);
	tex_lower1 = new TextureBox(this, 64, 64, "_none_", 1, 2.0f);
	label_upper1 = new wxStaticText(this, -1, "", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE);
	label_middle1 = new wxStaticText(this, -1, "", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE);
	label_lower1 = new wxStaticText(this, -1, "", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE);

	vbox = new wxBoxSizer(wxVERTICAL);
	box->Add(vbox);
	vbox->Add(new wxStaticText(this, -1, "Upper", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE), 0, wxEXPAND);
	vbox->Add(tex_upper1, 0, wxALIGN_CENTRE|wxEXPAND|wxTOP, 4);
	vbox->Add(label_upper1, 0, wxEXPAND|wxTOP, 4);

	vbox = new wxBoxSizer(wxVERTICAL);
	box->Add(vbox);
	vbox->Add(new wxStaticText(this, -1, "Middle", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE), 0, wxEXPAND);
	vbox->Add(tex_middle1, 0, wxALIGN_CENTRE|wxEXPAND|wxTOP, 4);
	vbox->Add(label_middle1, 0, wxEXPAND|wxTOP, 4);

	vbox = new wxBoxSizer(wxVERTICAL);
	box->Add(vbox);
	vbox->Add(new wxStaticText(this, -1, "Lower", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE), 0, wxEXPAND);
	vbox->Add(tex_lower1, 0, wxALIGN_CENTRE|wxEXPAND|wxTOP, 4);
	vbox->Add(label_lower1, 0, wxEXPAND|wxTOP, 4);

	// Back side
	back_frame = new wxStaticBox(this, -1, "Back Side");
	back_frame->SetFont(def_font);
	box = new wxStaticBoxSizer(back_frame, wxHORIZONTAL);
	box->SetMinSize(192, -1);
	hbox->Add(box, 0, wxEXPAND);

	tex_upper2 = new TextureBox(this, 64, 64, "_none_", 1, 2.0f);
	tex_middle2 = new TextureBox(this, 64, 64, "_none_", 1, 2.0f);
	tex_lower2 = new TextureBox(this, 64, 64, "_none_", 1, 2.0f);
	label_upper2 = new wxStaticText(this, -1, "", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE);
	label_middle2 = new wxStaticText(this, -1, "", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE);
	label_lower2 = new wxStaticText(this, -1, "", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE);

	vbox = new wxBoxSizer(wxVERTICAL);
	box->Add(vbox);
	vbox->Add(new wxStaticText(this, -1, "Upper", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE), 0, wxEXPAND);
	vbox->Add(tex_upper2, 0, wxALIGN_CENTRE|wxEXPAND|wxTOP, 4);
	vbox->Add(label_upper2, 0, wxEXPAND|wxTOP, 4);

	vbox = new wxBoxSizer(wxVERTICAL);
	box->Add(vbox);
	vbox->Add(new wxStaticText(this, -1, "Middle", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE), 0, wxEXPAND);
	vbox->Add(tex_middle2, 0, wxALIGN_CENTRE|wxEXPAND|wxTOP, 4);
	vbox->Add(label_middle2, 0, wxEXPAND|wxTOP, 4);

	vbox = new wxBoxSizer(wxVERTICAL);
	box->Add(vbox);
	vbox->Add(new wxStaticText(this, -1, "Lower", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTRE), 0, wxEXPAND);
	vbox->Add(tex_lower2, 0, wxALIGN_CENTRE|wxEXPAND|wxTOP, 4);
	vbox->Add(label_lower2, 0, wxEXPAND|wxTOP, 4);

	SetSizeHints(-1, 128, -1, 128);
}

LineInfoBar::~LineInfoBar()
{
}

void LineInfoBar::update()
{
	if (hilight_item == -1)
	{
		main_frame->SetLabel("No Line Hilighted");
		label_length->SetLabel("Length:");
		label_front->SetLabel("Front Sector:");
		label_back->SetLabel("Back Sector:");
		label_special->SetLabel("Special:");
		label_offsets1->SetLabel("Front Offsets:");
		label_offsets2->SetLabel("Back Offsets:");		

		front_frame->SetLabel("Front Side");
		label_upper1->SetLabel("");
		label_middle1->SetLabel("");
		label_lower1->SetLabel("");
		tex_upper1->set_texture("_none_");
		tex_middle1->set_texture("_none_");
		tex_lower1->set_texture("_none_");

		back_frame->SetLabel("Back Side");

		if (map.hexen)
		{
			label_arg1->SetLabel("Arg 1:");
			label_arg2->SetLabel("Arg 2:");
			label_arg3->SetLabel("Arg 3:");
			label_arg4->SetLabel("Arg 4:");
			label_arg5->SetLabel("Arg 5:");
			label_tag->SetLabel("Trigger:");
		}
		else
		{
			label_arg1->SetLabel("");
			label_arg2->SetLabel("");
			label_arg3->SetLabel("");
			label_arg4->SetLabel("");
			label_arg5->SetLabel("");
			label_tag->SetLabel("Sector Tag:");
		}

		return;
	}

	// Main frame
	main_frame->SetLabel(wxString::Format("Line #%d", hilight_item));
	label_length->SetLabel(wxString::Format("Length: %d", round(map.l_getrect(hilight_item).length())));
	label_front->SetLabel(wxString::Format("Front Sector: %d", map.l_getsector1(hilight_item)));
	label_back->SetLabel(wxString::Format("Back Sector: %d", map.l_getsector2(hilight_item)));
	label_special->SetLabel(wxString::Format("Special: %d (%s)",
											map.lines[hilight_item]->type,
											get_special(map.lines[hilight_item]->type)->name.c_str()));

	if (!map.hexen)
		label_tag->SetLabel(wxString::Format("Sector Tag: %d", map.lines[hilight_item]->sector_tag));
	else
		label_tag->SetLabel(wxString::Format("Trigger: %s", map.lines[hilight_item]->get_trigger_string().c_str()));

	if (map.hexen)
	{
		action_special_t* as = get_special(map.lines[hilight_item]->type);

		argtype_t *at = get_arg_type(as->arg_types[0]);
		if (at)
			label_arg1->SetLabel(wxString::Format("%s: %d (%s)", as->args[0].c_str(), map.lines[hilight_item]->args[0], at->get_name(map.lines[hilight_item]->args[0]).c_str()));
		else
			label_arg1->SetLabel(wxString::Format("%s: %d", as->args[0].c_str(), map.lines[hilight_item]->args[0]));

		at = get_arg_type(as->arg_types[1]);
		if (at)
			label_arg2->SetLabel(wxString::Format("%s: %d (%s)", as->args[1].c_str(), map.lines[hilight_item]->args[1], at->get_name(map.lines[hilight_item]->args[1]).c_str()));
		else
			label_arg2->SetLabel(wxString::Format("%s: %d", as->args[1].c_str(), map.lines[hilight_item]->args[1]));

		at = get_arg_type(as->arg_types[2]);
		if (at)
			label_arg3->SetLabel(wxString::Format("%s: %d (%s)", as->args[2].c_str(), map.lines[hilight_item]->args[2], at->get_name(map.lines[hilight_item]->args[2]).c_str()));
		else
			label_arg3->SetLabel(wxString::Format("%s: %d", as->args[2].c_str(), map.lines[hilight_item]->args[2]));

		at = get_arg_type(as->arg_types[3]);
		if (at)
			label_arg4->SetLabel(wxString::Format("%s: %d (%s)", as->args[3].c_str(), map.lines[hilight_item]->args[3], at->get_name(map.lines[hilight_item]->args[3]).c_str()));
		else
			label_arg4->SetLabel(wxString::Format("%s: %d", as->args[3].c_str(), map.lines[hilight_item]->args[3]));

		at = get_arg_type(as->arg_types[4]);
		if (at)
			label_arg5->SetLabel(wxString::Format("%s: %d (%s)", as->args[4].c_str(), map.lines[hilight_item]->args[4], at->get_name(map.lines[hilight_item]->args[4]).c_str()));
		else
			label_arg5->SetLabel(wxString::Format("%s: %d", as->args[4].c_str(), map.lines[hilight_item]->args[4]));
	}
			

	// Front side frame
	sidedef_t *side = map.l_getside(hilight_item, 1);
	if (side)
	{
		front_frame->SetLabel(parse_string("Front Side: %d", map.lines[hilight_item]->side1).c_str());

		// Upper texture
		label_upper1->SetLabel(side->tex_upper.c_str());
		if (map.l_needsuptex(hilight_item, 1))
			tex_upper1->set_texture(side->tex_upper, 1);
		else
			tex_upper1->set_texture("_none_");

		// Middle texture
		label_middle1->SetLabel(side->tex_middle.c_str());
		if (map.l_needsmidtex(hilight_item, 1))
			tex_middle1->set_texture(side->tex_middle, 1);
		else
			tex_middle1->set_texture("_none_");

		// Lower texture
		label_lower1->SetLabel(side->tex_lower.c_str());
		if (map.l_needslotex(hilight_item, 1))
			tex_lower1->set_texture(side->tex_lower, 1);
		else
			tex_lower1->set_texture("_none_");

		label_offsets1->SetLabel(parse_string("Front Offsets: (%d, %d)", side->x_offset, side->y_offset).c_str());
	}
	else
	{
		front_frame->SetLabel("Front Side");
		tex_upper1->set_texture("_none_");
		tex_middle1->set_texture("_none_");
		tex_lower1->set_texture("_none_");
		label_upper1->SetLabel("");
		label_middle1->SetLabel("");
		label_lower1->SetLabel("");
		label_offsets1->SetLabel("Front Offsets: (None)");
	}

	// Back side frame
	side = map.l_getside(hilight_item, 2);
	if (side)
	{
		back_frame->SetLabel(parse_string("Back Side: %d", map.lines[hilight_item]->side2).c_str());

		// Upper texture
		label_upper2->SetLabel(side->tex_upper.c_str());
		if (map.l_needsuptex(hilight_item, 2))
			tex_upper2->set_texture(side->tex_upper, 1);
		else
			tex_upper2->set_texture("_none_");

		// Middle texture
		label_middle2->SetLabel(side->tex_middle.c_str());
		if (map.l_needsmidtex(hilight_item, 2))
			tex_middle2->set_texture(side->tex_middle, 1);
		else
			tex_middle2->set_texture("_none_");

		// Lower texture
		label_lower2->SetLabel(side->tex_lower.c_str());
		if (map.l_needslotex(hilight_item, 2))
			tex_lower2->set_texture(side->tex_lower, 1);
		else
			tex_lower2->set_texture("_none_");

		label_offsets2->SetLabel(parse_string("Back Offsets: (%d, %d)", side->x_offset, side->y_offset).c_str());
	}
	else
	{
		back_frame->SetLabel("Back Side");
		tex_upper2->set_texture("_none_");
		tex_middle2->set_texture("_none_");
		tex_lower2->set_texture("_none_");
		label_upper2->SetLabel("");
		label_middle2->SetLabel("");
		label_lower2->SetLabel("");
		label_offsets2->SetLabel("Back Offsets: (None)");
	}

	Layout();
}
