
#include "main.h"
#include "line_edit.h"
#include "map.h"
#include "tex_browser.h"
#include "special_select.h"
#include "checks.h"
#include "args_edit.h"
#include "action_special.h"

#include <wx/list.h>

extern Map map;
extern int hilight_item;
extern vector<int> selected_items;

LineEditor::LineEditor()
:	wxDialog(NULL, -1, "Edit Linedef", wxDefaultPosition, wxDefaultSize)
{
	wxBoxSizer *m_vbox = new wxBoxSizer(wxVERTICAL);
	SetSizer(m_vbox);

	tabs = new wxNotebook(this, -1);
	m_vbox->Add(tabs, 1, wxEXPAND|wxALL, 4);

	props_page = new LinePropertiesPage(tabs);
	tabs->AddPage(props_page, "Properties");

	side1_page = new SidePropertiesPage(tabs);
	tabs->AddPage(side1_page, "Front Side");

	side2_page = new SidePropertiesPage(tabs, 2);
	tabs->AddPage(side2_page, "Back Side");

	m_vbox->Add(CreateButtonSizer(wxOK|wxCANCEL), 0, wxEXPAND|wxALL, 4);

	Layout();
	SetBestFittingSize();
	Center();
}

LineEditor::~LineEditor()
{
}

void LineEditor::apply_changes()
{
	props_page->apply_changes();
	side1_page->apply_changes();
	side2_page->apply_changes();
	map.change_level(MC_LINES);
}

LinePropertiesPage::LinePropertiesPage(wxWindow *parent)
:	wxPanel(parent, -1)
{
	wxBoxSizer *m_vbox = new wxBoxSizer(wxVERTICAL);
	SetSizer(m_vbox);

	// Flags frame
	wxStaticBox *frame = new wxStaticBox(this, -1, "Flags");
	wxStaticBoxSizer *box = new wxStaticBoxSizer(frame, wxHORIZONTAL);
	m_vbox->Add(box, 0, wxEXPAND|wxALL, 4);

	// Flags
	wxBoxSizer *vbox = NULL;

	int flags_per_col = 6;
	if (map.line_flags.size() > 12)
		flags_per_col = (map.line_flags.size() / 2) + 1;

	int f = 0;
	int cols = (map.line_flags.size() / flags_per_col) + 1;
	for (int a = 0; a < cols; a++)
	{
		if (f >= map.line_flags.size())
			continue;

		vbox = new wxBoxSizer(wxVERTICAL);
		box->Add(vbox, 1, wxEXPAND);

		for (int b = 0; b < flags_per_col; b++)
		{
			if (f >= map.line_flags.size())
				continue;

			setup_flag_checkbox(map.line_flags[f].value, map.line_flags[f].name.c_str(), vbox);
			f++;
		}
	}

	/*
	wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
	box->Add(vbox, 1, wxEXPAND);

	setup_flag_checkbox(LINE_IMPASSIBLE, "Impassible", vbox);
	setup_flag_checkbox(LINE_BLOCKMONSTERS, "Blocks Monsters", vbox);
	setup_flag_checkbox(LINE_TWOSIDED, "Two Sided", vbox);
	setup_flag_checkbox(LINE_UPPERUNPEGGED, "Upper Texture Unpegged", vbox);
	setup_flag_checkbox(LINE_LOWERUNPEGGED, "Lower Texture Unpegged", vbox);
	setup_flag_checkbox(LINE_SECRET, "Secret (Shown as 1S on Map)", vbox);

	vbox = new wxBoxSizer(wxVERTICAL);
	box->Add(vbox, 1, wxEXPAND);

	setup_flag_checkbox(LINE_BLOCKSOUND, "Blocks Sound", vbox);
	setup_flag_checkbox(LINE_NOTONMAP, "Doesn't Show On Map", vbox);
	setup_flag_checkbox(LINE_STARTONMAP, "Starts On Map", vbox);

	if (map.hexen)
	{
		setup_flag_checkbox(LINE_REPEATABLE, "Repeatable Special", vbox);
		setup_flag_checkbox(LINE_MONSTERCANACT, "Monsters Can Activate", vbox);
		setup_flag_checkbox(LINE_BLOCKALL, "Blocks Everything", vbox);
	}
	*/

	// Properties frame
	frame = new wxStaticBox(this, -1, "Properties");
	box = new wxStaticBoxSizer(frame, wxVERTICAL);
	m_vbox->Add(box, 0, wxEXPAND|wxALL, 4);

	if (map.hexen)
	{
		// Action special
		wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
		box->Add(hbox, 0, wxEXPAND);

		hbox->Add(new wxStaticText(this, -1, "Action Special:"), 1, wxALIGN_CENTER_VERTICAL);
		entry_special = new wxTextCtrl(this, LP_ENTRY_SPECIAL, "", wxDefaultPosition, wxSize(48, -1));
		hbox->Add(entry_special, 0, wxEXPAND|wxRIGHT, 4);
		btn_changespecial = new wxButton(this, LP_BTN_CHANGESPECIAL, "Change");
		hbox->Add(btn_changespecial, 0, wxEXPAND|wxRIGHT, 4);
		btn_editargs = new wxButton(this, LP_BTN_EDITARGS, "Edit Args");
		hbox->Add(btn_editargs, 0, wxEXPAND);

		label_special = new wxStaticText(this, -1, "Special:");
		box->Add(label_special, 0, wxEXPAND|wxTOP, 2);

		// Trigger
		hbox = new wxBoxSizer(wxHORIZONTAL);
		box->Add(hbox, 0, wxEXPAND|wxTOP, 4);

		wxString triggers[] =
		{
			"",
			"Player Crosses Line",
			"Player Uses Line",
			"Monster Crosses Line",
			"Projectile Hits Line",
			"Player Bumps Line",
			"Projectile Crosses Line",
			"Player Uses Line (PassThru)",
			"Projectile Hits OR Crosses Line",
		};

		hbox->Add(new wxStaticText(this, -1, "Activation Trigger:"), 1, wxALIGN_CENTER_VERTICAL);
		combo_trigger = new wxChoice(this, -1, wxDefaultPosition, wxDefaultSize, 9, triggers);
		hbox->Add(combo_trigger, 0, wxEXPAND);
	}
	else
	{
		// Action special
		wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
		box->Add(hbox, 0, wxEXPAND);

		label_special = new wxStaticText(this, -1, "Special:");
		hbox->Add(label_special, 1, wxALIGN_CENTER_VERTICAL);
		entry_special = new wxTextCtrl(this, LP_ENTRY_SPECIAL, "", wxDefaultPosition, wxSize(32, -1));
		hbox->Add(entry_special, 0, wxEXPAND|wxRIGHT, 4);
		btn_changespecial = new wxButton(this, LP_BTN_CHANGESPECIAL, "Change");
		hbox->Add(btn_changespecial, 0, wxEXPAND);

		// Sector tag
		hbox = new wxBoxSizer(wxHORIZONTAL);
		box->Add(hbox, 0, wxEXPAND|wxTOP, 4);

		hbox->Add(new wxStaticText(this, -1, "Sector Tag:"), 1, wxALIGN_CENTER_VERTICAL);
		entry_tag = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxSize(32, -1));
		hbox->Add(entry_tag, 0, wxEXPAND|wxRIGHT, 4);
		btn_getfreetag = new wxButton(this, LP_BTN_GETFREETAG, "Free Tag");
		hbox->Add(btn_getfreetag, 0, wxEXPAND);
	}

	setup_widgets();

	Layout();
}

LinePropertiesPage::~LinePropertiesPage()
{
}

void LinePropertiesPage::setup_flag_checkbox(WORD flag, wxString label, wxSizer *box)
{
	FlagCheckBox *flag_cb = new FlagCheckBox(this, flag, label);
	flags.push_back(flag_cb);
	box->Add(flag_cb, 0, wxBOTTOM, 2);
}

void LinePropertiesPage::setup_widgets()
{
	// Single item
	if (selected_items.size() == 0 && hilight_item != -1)
	{
		// Flags
		for (int a = 0; a < flags.size(); a++)
		{
			if (map.lines[hilight_item]->flags & flags[a]->flag)
				flags[a]->Set3StateValue(wxCHK_CHECKED);
		}

		// Properties
		entry_special->SetValue(parse_string("%d", map.lines[hilight_item]->type).c_str());

		if (!map.hexen)
			entry_tag->SetValue(parse_string("%d", map.lines[hilight_item]->sector_tag).c_str());
		else
		{
			combo_trigger->Select(map.lines[hilight_item]->get_trigger() + 1);

			memcpy(args, map.lines[hilight_item]->args, 5);
			memset(args_consistent, 1, 5);
		}

		return;
	}

	// Multiple items
	if (selected_items.size() > 0)
	{
		// Flags
		for (int a = 0; a < flags.size(); a++)
		{
			if (map.lines[selected_items[0]]->flags & flags[a]->flag)
				flags[a]->Set3StateValue(wxCHK_CHECKED);
		}

		// Properties
		entry_special->SetValue(parse_string("%d", map.lines[selected_items[0]]->type).c_str());

		if (!map.hexen)
			entry_tag->SetValue(parse_string("%d", map.lines[selected_items[0]]->sector_tag).c_str());
		else
		{
			combo_trigger->Select(map.lines[selected_items[0]]->get_trigger() + 1);

			memcpy(args, map.lines[selected_items[0]]->args, 5);
			memset(args_consistent, 1, 5);
		}

		// Check for inconsistency
		for (int a = 0; a < selected_items.size(); a++)
		{
			for (int b = 0; b < flags.size(); b++)
			{
				if (flags[b]->Get3StateValue() == wxCHK_UNDETERMINED)
					continue;

				bool on = map.lines[selected_items[a]]->flags & flags[b]->flag;
				if (on && flags[b]->Get3StateValue() == wxCHK_UNCHECKED ||
					!on && flags[b]->Get3StateValue() == wxCHK_CHECKED)
					flags[b]->Set3StateValue(wxCHK_UNDETERMINED);
			}

			if (map.lines[selected_items[a]]->type != map.lines[selected_items[0]]->type)
				entry_special->SetValue("");

			if (!map.hexen && map.lines[selected_items[a]]->sector_tag != map.lines[selected_items[0]]->sector_tag)
				entry_tag->SetValue("");

			if (map.hexen && map.lines[selected_items[a]]->get_trigger() != map.lines[selected_items[0]]->get_trigger())
				combo_trigger->Select(0);

			if (map.hexen)
			{
				for (int b = 0; b < 5; b++)
				{
					if (map.lines[selected_items[a]]->args[b] != map.lines[selected_items[0]]->args[b])
						args_consistent[b] = false;
				}
			}
		}
	}
}

void LinePropertiesPage::apply_changes()
{
	vector<int> items;

	// Get items to modify
	if (selected_items.size() == 0 && hilight_item != -1)
		items.push_back(hilight_item);

	if (selected_items.size() > 0)
	{
		for (int a = 0; a < selected_items.size(); a++)
			items.push_back(selected_items[a]);
	}

	// Flags
	for (int a = 0; a < items.size(); a++)
	{
		for (int b = 0; b < flags.size(); b++)
			flags[b]->apply_flag(&map.lines[items[a]]->flags);
	}

	// Action special
	if (entry_special->GetValue() != "")
	{
		for (int a = 0; a < items.size(); a++)
			map.lines[items[a]]->type = atoi(entry_special->GetValue());
	}

	if (!map.hexen)
	{
		// Sector tag
		if (entry_tag->GetValue() != "")
		{
			for (int a = 0; a < items.size(); a++)
				map.lines[items[a]]->sector_tag = atoi(entry_tag->GetValue());
		}
	}
	else
	{
		// Trigger
		if (combo_trigger->GetSelection() > 0)
		{
			for (int a = 0; a < items.size(); a++)
				map.lines[items[a]]->set_trigger(combo_trigger->GetSelection() - 1);
		}

		// Args
		for (int a = 0; a < 5; a++)
		{
			if (args_consistent[a])
			{
				for (int b = 0; b < items.size(); b++)
					map.lines[items[b]]->args[a] = args[a];
			}
		}
	}
}

BEGIN_EVENT_TABLE(LinePropertiesPage, wxPanel)
	EVT_BUTTON(LP_BTN_CHANGESPECIAL, LinePropertiesPage::change_special_clicked)
	EVT_BUTTON(LP_BTN_GETFREETAG, LinePropertiesPage::get_free_tag_clicked)
	EVT_BUTTON(LP_BTN_EDITARGS, LinePropertiesPage::edit_args_clicked)
	EVT_TEXT(LP_ENTRY_SPECIAL, LinePropertiesPage::entry_special_changed)
END_EVENT_TABLE()

void LinePropertiesPage::change_special_clicked(wxCommandEvent &event)
{
	int val = 0;
	if (entry_special->GetValue() != "")
		val = atoi(entry_special->GetValue());

	SpecialSelectDialog sd(val);
	if (sd.ShowModal() == wxID_OK)
	{
		if (sd.get_special() != -1)
			entry_special->SetValue(parse_string("%d", sd.get_special()).c_str());
	}
}

void LinePropertiesPage::get_free_tag_clicked(wxCommandEvent &event)
{
	int tag = get_free_tag();
	entry_tag->SetValue(wxString::Format("%d", tag));
}

void LinePropertiesPage::edit_args_clicked(wxCommandEvent &event)
{
	int type = 0;
	if (entry_special->GetValue() != "")
		type = atoi(entry_special->GetValue());

	action_special_t *as = get_special(type);

	ArgsEditor ae(args, as->args, as->arg_types, args_consistent);
	if (ae.ShowModal() == wxID_OK)
		ae.apply_changes(args, args_consistent);
}

void LinePropertiesPage::entry_special_changed(wxCommandEvent &event)
{
	if (entry_special->GetValue() == "")
		label_special->SetLabel("Special:");
	else
	{
		action_special_t *as = get_special(atoi(entry_special->GetValue()));
		label_special->SetLabel(wxString::Format("Special: %s", as->name.c_str()));
	}
}

FlagCheckBox::FlagCheckBox(wxWindow *parent, int flag, wxString label)
:	wxCheckBox(parent, -1, label, wxDefaultPosition, wxDefaultSize, wxCHK_3STATE)
{
	this->flag = flag;
}

FlagCheckBox::~FlagCheckBox()
{
}

void FlagCheckBox::apply_flag(WORD *flags)
{
	if (Get3StateValue() == wxCHK_UNCHECKED)
		*flags = (*flags & ~flag);

	if (Get3StateValue() == wxCHK_CHECKED)
		*flags |= flag;
}

SidePropertiesPage::SidePropertiesPage(wxWindow *parent, int side)
:	wxPanel(parent, -1)
{
	this->side = side;

	wxBoxSizer *m_vbox = new wxBoxSizer(wxVERTICAL);
	SetSizer(m_vbox);

	// Textures frame
	wxStaticBox *frame = new wxStaticBox(this, -1, "Textures");
	wxStaticBoxSizer *box = new wxStaticBoxSizer(frame, wxHORIZONTAL);
	m_vbox->Add(box, 0, wxEXPAND|wxALL, 4);

	// Upper
	wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
	box->Add(vbox, 1, wxEXPAND);
	box->AddSpacer(4);

	vbox->Add(new wxStaticText(this, -1, "Upper", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTER), 0, wxEXPAND);
	entry_upper = new wxTextCtrl(this, SP_ENTRY_UPPER, "", wxDefaultPosition, wxDefaultSize, wxTE_CENTRE);
	tex_upper = new ClickableTexBox(this, 96, 96, entry_upper, "_none_", 1, 5.0f, SP_TEX_UPPER);
	tex_upper->Enable(true);
	vbox->Add(tex_upper, 0, wxEXPAND|wxTOP, 4);
	vbox->Add(entry_upper, 0, wxEXPAND|wxTOP, 4);

	// Middle
	vbox = new wxBoxSizer(wxVERTICAL);
	box->Add(vbox, 1, wxEXPAND);
	box->AddSpacer(4);

	vbox->Add(new wxStaticText(this, -1, "Middle", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTER), 0, wxEXPAND);
	entry_middle = new wxTextCtrl(this, SP_ENTRY_MIDDLE, "", wxDefaultPosition, wxDefaultSize, wxTE_CENTRE);
	tex_middle = new ClickableTexBox(this, 96, 96, entry_middle, "_none_", 1, 5.0f, SP_TEX_MIDDLE);
	tex_middle->Enable(true);
	vbox->Add(tex_middle, 0, wxEXPAND|wxTOP, 4);
	vbox->Add(entry_middle, 0, wxEXPAND|wxTOP, 4);

	// Lower
	vbox = new wxBoxSizer(wxVERTICAL);
	box->Add(vbox, 1, wxEXPAND);

	vbox->Add(new wxStaticText(this, -1, "Lower", wxDefaultPosition, wxDefaultSize, wxALIGN_CENTER), 0, wxEXPAND);
	entry_lower = new wxTextCtrl(this, SP_ENTRY_LOWER, "", wxDefaultPosition, wxDefaultSize, wxTE_CENTRE);
	tex_lower = new ClickableTexBox(this, 96, 96, entry_lower, "_none_", 1, 5.0f, SP_TEX_LOWER);
	tex_lower->Enable(true);
	vbox->Add(tex_lower, 0, wxEXPAND|wxTOP, 4);
	vbox->Add(entry_lower, 0, wxEXPAND|wxTOP, 4);

	wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
	m_vbox->Add(hbox, 0, wxEXPAND);

	// Offsets frame
	frame = new wxStaticBox(this, -1, "Offsets");
	box = new wxStaticBoxSizer(frame, wxVERTICAL);
	hbox->Add(box, 1, wxEXPAND|wxALL, 4);

	// X
	wxBoxSizer *hbox2 = new wxBoxSizer(wxHORIZONTAL);
	box->Add(hbox2, 0, wxEXPAND);
	entry_xoff = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxSize(56, -1));
	hbox2->Add(new wxStaticText(this, -1, "X Offset:"), 0, wxALIGN_CENTER_VERTICAL);
	hbox2->AddStretchSpacer();
	hbox2->Add(entry_xoff, 0, wxEXPAND|wxALIGN_RIGHT);

	// Y
	hbox2 = new wxBoxSizer(wxHORIZONTAL);
	box->Add(hbox2, 0, wxEXPAND|wxTOP, 4);
	entry_yoff = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxSize(56, -1));
	hbox2->Add(new wxStaticText(this, -1, "Y Offset:"), 0, wxALIGN_CENTER_VERTICAL);
	hbox2->AddStretchSpacer();
	hbox2->Add(entry_yoff, 0, wxEXPAND|wxALIGN_RIGHT);

	// Side frame
	frame = new wxStaticBox(this, -1, "Side");
	box = new wxStaticBoxSizer(frame, wxVERTICAL);
	hbox->Add(box, 1, wxEXPAND|wxALL, 4);

	// Sector reference
	hbox2 = new wxBoxSizer(wxHORIZONTAL);
	box->Add(hbox2, 0, wxEXPAND);
	entry_sector = new wxTextCtrl(this, -1, "", wxDefaultPosition, wxSize(56, -1));
	hbox2->Add(new wxStaticText(this, -1, "Sector Reference:"), 0, wxALIGN_CENTER_VERTICAL);
	hbox2->AddStretchSpacer();
	hbox2->Add(entry_sector, 0, wxEXPAND|wxALIGN_RIGHT);

	// Create/Delete buttons
	hbox2 = new wxBoxSizer(wxHORIZONTAL);
	box->Add(hbox2, 0, wxEXPAND|wxTOP, 4);
	btn_create = new wxButton(this, -1, "Create");
	btn_delete = new wxButton(this, -1, "Delete");
	//hbox2->AddStretchSpacer();
	hbox2->Add(btn_create, 0, wxEXPAND);
	hbox2->Add(btn_delete, 0, wxEXPAND|wxLEFT, 4);

	setup_widgets();
}

SidePropertiesPage::~SidePropertiesPage()
{
}

void SidePropertiesPage::setup_widgets()
{
	bool missingside = false;

	// Get list of relevant sidedefs
	vector<sidedef_t*> sides;
	if (selected_items.size() == 0 && hilight_item != -1)
	{
		if (map.l_getside(hilight_item, side))
			sides.push_back(map.l_getside(hilight_item, side));
		else
			missingside = true;
	}

	if (selected_items.size() > 0)
	{
		for (int a = 0; a < selected_items.size(); a++)
		{
			if (map.l_getside(selected_items[a], side))
				sides.push_back(map.l_getside(selected_items[a], side));
			else
				missingside = true;
		}
	}

	// If there are no sides in the list, disable all widgets except the 'create' button
	if (sides.size() == 0)
	{
		wxList list = GetChildren();
		wxNode *node = list.GetFirst();

		while (node)
		{
			((wxWindow*)(node->GetData()))->Enable(false);
			node = node->GetNext();
		}

		btn_create->Enable(true);
		return;
	}
	else if (!missingside)
		btn_create->Enable(false);

	// Upper texture
	entry_upper->SetValue(sides[0]->tex_upper.c_str());
	if (sides[0]->tex_upper == "-")
		tex_upper->set_texture("_none_");
	else
		tex_upper->set_texture(sides[0]->tex_upper, 1);

	// Middle texture
	entry_middle->SetValue(sides[0]->tex_middle.c_str());
	if (sides[0]->tex_middle == "-")
		tex_middle->set_texture("_none_");
	else
		tex_middle->set_texture(sides[0]->tex_middle, 1);

	// Lower texture
	entry_lower->SetValue(sides[0]->tex_lower.c_str());
	if (sides[0]->tex_lower == "-")
		tex_lower->set_texture("_none_");
	else
		tex_lower->set_texture(sides[0]->tex_lower, 1);

	// Offsets
	entry_xoff->SetValue(parse_string("%d", sides[0]->x_offset).c_str());
	entry_yoff->SetValue(parse_string("%d", sides[0]->y_offset).c_str());

	// Sector ref
	entry_sector->SetValue(parse_string("%d", sides[0]->sector).c_str());

	// Check for inconsistency
	for (int a = 0; a < sides.size(); a++)
	{
		if (sides[a]->tex_upper != sides[0]->tex_upper)
		{
			entry_upper->SetValue("");
			tex_upper->set_texture("_none_", 1);
		}

		if (sides[a]->tex_middle != sides[0]->tex_middle)
		{
			entry_middle->SetValue("");
			tex_middle->set_texture("_none_", 1);
		}

		if (sides[a]->tex_lower != sides[0]->tex_lower)
		{
			entry_lower->SetValue("");
			tex_lower->set_texture("_none_", 1);
		}

		if (sides[a]->x_offset != sides[0]->x_offset)
			entry_xoff->SetValue("");

		if (sides[a]->y_offset != sides[0]->y_offset)
			entry_yoff->SetValue("");

		if (sides[a]->sector != sides[0]->sector)
			entry_sector->SetValue("");
	}
}

void SidePropertiesPage::apply_changes()
{
	// Get list of relevant sidedefs
	vector<sidedef_t*> sides;
	if (selected_items.size() == 0 && hilight_item != -1)
	{
		if (map.l_getside(hilight_item, side))
			sides.push_back(map.l_getside(hilight_item, side));
	}

	if (selected_items.size() > 0)
	{
		for (int a = 0; a < selected_items.size(); a++)
		{
			if (map.l_getside(selected_items[a], side))
				sides.push_back(map.l_getside(selected_items[a], side));
		}
	}

	for (int a = 0; a < sides.size(); a++)
	{
		// Textures
		if (entry_upper->GetValue() != "")
			sides[a]->tex_upper = entry_upper->GetValue();

		if (entry_middle->GetValue() != "")
			sides[a]->tex_middle = entry_middle->GetValue();

		if (entry_lower->GetValue() != "")
			sides[a]->tex_lower = entry_lower->GetValue();

		// Offsets
		if (entry_xoff->GetValue() != "")
			sides[a]->x_offset = atoi(entry_xoff->GetValue());

		if (entry_yoff->GetValue() != "")
			sides[a]->y_offset = atoi(entry_yoff->GetValue());

		// Sector ref
		if (entry_sector->GetValue() != "")
			sides[a]->sector = atoi(entry_sector->GetValue());
	}
}

void SidePropertiesPage::upper_tex_entry_changed(wxCommandEvent &event)
{
	wxString str = entry_upper->GetValue();
	str.UpperCase();
	tex_upper->set_texture((string)str, 1);
}

void SidePropertiesPage::middle_tex_entry_changed(wxCommandEvent &event)
{
	wxString str = entry_middle->GetValue();
	str.UpperCase();
	tex_middle->set_texture((string)str, 1);
}

void SidePropertiesPage::lower_tex_entry_changed(wxCommandEvent &event)
{
	wxString str = entry_lower->GetValue();
	str.UpperCase();
	tex_lower->set_texture((string)str, 1);
}

void SidePropertiesPage::upper_tex_clicked(wxMouseEvent &event)
{
}

BEGIN_EVENT_TABLE(SidePropertiesPage, wxPanel)
	EVT_TEXT(SP_ENTRY_UPPER, SidePropertiesPage::upper_tex_entry_changed)
	EVT_TEXT(SP_ENTRY_MIDDLE, SidePropertiesPage::middle_tex_entry_changed)
	EVT_TEXT(SP_ENTRY_LOWER, SidePropertiesPage::lower_tex_entry_changed)
END_EVENT_TABLE()
