
#include <wx/colordlg.h>
#include <wx/dir.h>

#include "main.h"
#include "prefs_dialog.h"
#include "editor_window.h"
#include "colours.h"

extern rgba_t col_background;
extern rgba_t col_hilight;
extern rgba_t col_selection;
extern rgba_t col_moving;
extern rgba_t col_tagged;
extern rgba_t col_vertex;
extern rgba_t col_line_solid;
extern rgba_t col_line_2s;
extern rgba_t col_line_monster;
extern rgba_t col_line_special;
extern rgba_t col_line_nofirst;
extern rgba_t col_linedraw;
extern rgba_t col_selbox;
extern rgba_t col_selbox_line;
extern rgba_t col_grid;
extern rgba_t col_64grid;
extern rgba_t col_crosshair;
extern rgba_t col_3d_crosshair;
extern rgba_t col_3d_hilight;
extern rgba_t col_3d_hilight_line;

extern EditorWindow *editor_window;

EXTERN_CVAR(String, col_config)

ColourBox::ColourBox(wxWindow *parent, rgba_t *colour)
:	wxPanel(parent, -1, wxDefaultPosition, wxSize(32, 24), wxSUNKEN_BORDER)
{
	//this->colour.Set(colour->r, colour->g, colour->b);
	//alpha = colour->fa();
	rgba = colour;
}

ColourBox::~ColourBox()
{
}

BEGIN_EVENT_TABLE(ColourBox, wxPanel)
	EVT_PAINT(ColourBox::paint)
	EVT_LEFT_DOWN(ColourBox::clicked)
	EVT_RIGHT_DOWN(ColourBox::clicked2)
END_EVENT_TABLE()

void ColourBox::paint(wxPaintEvent &event)
{
	wxPaintDC dc(this);

	dc.BeginDrawing();

	dc.SetBrush(wxBrush(wxColour(rgba->r, rgba->g, rgba->b)));
	dc.DrawRectangle(0, 0, GetClientSize().x, GetClientSize().y);

	int a_point = rgba->fa() * (GetClientSize().x - 2);

	dc.SetBrush(wxBrush(wxColour(0, 0, 0)));
	dc.DrawRectangle(0, 0, GetClientSize().x, 4);

	dc.SetBrush(wxBrush(wxColour(255, 255, 255)));
	dc.SetPen(*wxTRANSPARENT_PEN);
	dc.DrawRectangle(1, 1, a_point, 2);

	dc.EndDrawing();
}

void ColourBox::clicked(wxMouseEvent &event)
{
	wxColour col = wxGetColourFromUser(GetParent(), wxColour(rgba->r, rgba->g, rgba->b));

	if (col.Ok())
	{
		//colour = col;
		rgba->r = col.Red();
		rgba->g = col.Green();
		rgba->b = col.Blue();
		Refresh();
		editor_window->redraw_map(true, true);
	}
}

void ColourBox::clicked2(wxMouseEvent &event)
{
	// Popup a dialog with a slider control for alpha
	wxDialog dlg(NULL, -1, "Set Alpha", wxDefaultPosition, wxDefaultSize);
	wxBoxSizer *box = new wxBoxSizer(wxVERTICAL);
	dlg.SetSizer(box);
	wxSlider *slider = new wxSlider(&dlg, -1, rgba->a, 0, 255, wxDefaultPosition, wxDefaultSize, wxSL_HORIZONTAL);
	box->Add(slider, 1, wxEXPAND|wxALL, 4);
	box->Add(dlg.CreateButtonSizer(wxOK|wxCANCEL), 0, wxEXPAND|wxALL, 4);
	dlg.SetBestFittingSize();

	if (dlg.ShowModal() == wxID_OK)
	{
		rgba->a = slider->GetValue();
		Refresh();
		editor_window->redraw_map(true, true);
	}
}

ColourEdit::ColourEdit(wxWindow *parent, rgba_t *colour, string name)
:	wxPanel(parent, -1)
{
	col = colour;

	wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
	SetSizer(hbox);

	hbox->Add(new wxStaticText(this, -1, name.c_str()), 1, wxALIGN_CENTER_VERTICAL|wxRIGHT, 4);

	cbox_colour = new ColourBox(this, col);
	hbox->Add(cbox_colour, 0, wxEXPAND|wxRIGHT, 4);

	cb_additive = new wxCheckBox(this, 1, "Additive");
	hbox->Add(cb_additive, 0, wxALIGN_CENTER_VERTICAL);
	if (colour->blend == 1)
		cb_additive->SetValue(true);
}

ColourEdit::~ColourEdit()
{
}

BEGIN_EVENT_TABLE(ColourEdit, wxPanel)
	EVT_CHECKBOX(1, ColourEdit::cb_additive_changed)
END_EVENT_TABLE()

void ColourEdit::cb_additive_changed(wxCommandEvent &event)
{
	if (cb_additive->GetValue())
		col->blend = 1;
	else
		col->blend = 0;

	editor_window->redraw_map(true, true);
}

ColourPrefs::ColourPrefs(wxWindow *parent)
:	wxPanel(parent, -1)
{
	wxBoxSizer *m_vbox = new wxBoxSizer(wxVERTICAL);
	SetSizer(m_vbox);

	// Colour config frame
	wxStaticBox *frame = new wxStaticBox(this, -1, "Colour Configuration");
	wxStaticBoxSizer *box = new wxStaticBoxSizer(frame, wxHORIZONTAL);
	m_vbox->Add(box, 0, wxEXPAND|wxALL, 4);

	colour_configs = new wxChoice(this, CP_COMBO_CONFIGS, wxDefaultPosition, wxDefaultSize, 0, NULL);
	box->Add(colour_configs, 1, wxEXPAND);

	// Get colour configurations
	wxDir dir;
	//wxArrayString configs;
	int cur_config;

	if (dir.Open("./config/colours/"))
	{
		string def = col_config;
		wxString filename;
		bool cont = dir.GetFirst(&filename, "*.cfg");

		while (cont)
		{
			Tokenizer tokenizer;
			string path = "config/colours/";
			path += filename;
			tokenizer.open_file(path, 0, 0);

			string temp = tokenizer.get_token();

			if (temp == "colour_config") // If it's a valid SLADE game configuration
			{
				// Add the file path to the game config list
				config_paths.push_back(path);

				// And add the game name to the combo
				string name = tokenizer.get_token();
				colour_configs->Append(name.c_str());

				if (name == def)
					cur_config = colour_configs->GetCount() - 1;
			}

			cont = dir.GetNext(&filename);
		}

		colour_configs->SetSelection(cur_config);
	}

	// Colours frame
	frame = new wxStaticBox(this, -1, "Colours");
	box = new wxStaticBoxSizer(frame, wxVERTICAL);
	m_vbox->Add(box, 1, wxEXPAND|wxALL, 4);

	box->Add(new wxStaticText(this, -1, "Right click the colour boxes to set alpha"), 0, wxEXPAND|wxBOTTOM, 2);

	wxScrolledWindow *s_window = new wxScrolledWindow(this, -1, wxDefaultPosition, wxDefaultSize, wxVSCROLL|wxSUNKEN_BORDER);
	wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
	s_window->SetSizer(vbox);
	s_window->SetScrollRate(0, 4);
	box->Add(s_window, 1, wxEXPAND);

	ColourEdit *cedit = NULL;

	cedit = new ColourEdit(s_window, &col_background, "Background");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_grid, "Grid");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_64grid, "64 Grid");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_line_solid, "Solid Line");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_line_2s, "2-Sided Line");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_line_monster, "Monster Blocking Line");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_line_special, "Special Line");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_line_nofirst, "Invalid Line");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_hilight, "Hilighted Item");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_selection, "Selected Item");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_moving, "Moving Item");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_tagged, "Tagged Item");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_vertex, "Vertex");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_linedraw, "Draw Lines");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_selbox, "Selection Box");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_selbox_line, "Selection Box Outline");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_crosshair, "2d Mode Crosshair");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_3d_crosshair, "3d Mode Crosshair");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_3d_hilight, "3d Mode Hilight");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);
	cedit = new ColourEdit(s_window, &col_3d_hilight_line, "3d Mode Hilight Outline");
	vbox->Add(cedit, 0, wxEXPAND|wxALL, 4);

	// Buttons
	wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
	m_vbox->Add(hbox, 0, wxEXPAND|wxALL, 4);

	hbox->AddSpacer(1);

	btn_newconfig = new wxButton(this, -1, "New Config");
	hbox->Add(btn_newconfig, 0, wxEXPAND|wxRIGHT, 4);

	btn_saveconfig = new wxButton(this, -1, "Save Config");
	hbox->Add(btn_saveconfig, 0, wxEXPAND);
}

ColourPrefs::~ColourPrefs()
{
}

BEGIN_EVENT_TABLE(ColourPrefs, wxPanel)
	EVT_CHOICE(CP_COMBO_CONFIGS, ColourPrefs::combo_configs_changed)
END_EVENT_TABLE()

void ColourPrefs::combo_configs_changed(wxCommandEvent &event)
{
	string ccfg = colour_configs->GetStringSelection();
	//wxMessageBox(ccfg.c_str());
	set_colour_config((string)colour_configs->GetStringSelection());
	Refresh();
	editor_window->redraw_map(true, true);
}
