
#include "main.h"
#include "script_editor.h"
#include "map.h"

CVAR(String, path_acc, "acc", CVAR_SAVE)

extern Map map;
extern string app_path;

ScriptEditor::ScriptEditor(wxWindow *parent)
:	wxFrame(parent, -1, _T("Script Editor"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_FRAME_STYLE|wxFRAME_FLOAT_ON_PARENT)
{
	wxPanel	*back_panel = new wxPanel(this, -1);
	wxBoxSizer *m_vbox = new wxBoxSizer(wxVERTICAL);
	back_panel->SetSizer(m_vbox);

	text_script = new wxTextCtrl(back_panel, SE_TEXT_SCRIPT, "", wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE|wxTE_PROCESS_TAB|wxTE_RICH2);
	text_script->SetFont(wxFont(10, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL));
	text_script->SetValue(string((char*)map.scripts->Data(), map.scripts->Size()).c_str());
	m_vbox->Add(text_script, 1, wxEXPAND|wxALL, 4);

	wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
	m_vbox->Add(hbox, 0, wxEXPAND|wxALL, 4);

	hbox->AddStretchSpacer();

	btn_savechanges = new wxButton(back_panel, SE_BTN_SAVECHANGES, "Save Changes");
	hbox->Add(btn_savechanges, 0, wxRIGHT|wxEXPAND, 4);

	btn_compile = new wxButton(back_panel, SE_BTN_COMPILE, "Compile Script");
	hbox->Add(btn_compile, 0, wxEXPAND);

	SetIcon(wxIcon("res/slade.ico",  wxBITMAP_TYPE_ICO, -1, -1));
}

ScriptEditor::~ScriptEditor()
{
}

BEGIN_EVENT_TABLE(ScriptEditor, wxFrame)
	EVT_CLOSE(ScriptEditor::close)
	EVT_BUTTON(SE_BTN_SAVECHANGES, ScriptEditor::save_changes_clicked)
	EVT_BUTTON(SE_BTN_COMPILE, ScriptEditor::compile_clicked)
	EVT_TEXT(SE_TEXT_SCRIPT, ScriptEditor::text_changed)
END_EVENT_TABLE()

void ScriptEditor::close(wxCloseEvent &event)
{
	if (!event.CanVeto())
	{
		delete this;
		return;
	}

	Hide();
}

void ScriptEditor::save_changes_clicked(wxCommandEvent &event)
{
	string content = wx_to_str(text_script->GetValue());
	map.scripts->LoadData((BYTE*)content.c_str(), content.size());
}

void ScriptEditor::compile_clicked(wxCommandEvent &event)
{
	string o_path = parse_string("%ssladetemp.o", app_path.c_str());
	string acs_path = parse_string("%ssladetemp.acs", app_path.c_str());

	remove(o_path.c_str());

	// Save the script
	string content = wx_to_str(text_script->GetValue());
	map.scripts->LoadData((BYTE*)content.c_str(), content.size());
	map.scripts->DumpToFile(acs_path);

	// Compile it
	string path = path_acc;
	if (wxExecute(wxString::Format(_T("%s %s %s"), path.c_str(), acs_path.c_str(), o_path.c_str()), wxEXEC_SYNC) == -1)
	{
		wxLogMessage("Acc(.exe) not found! Please check the path (edit->preferences->paths)");
		return;
	}

	//system("acc sladetemp.acs sladetemp.o");

	// Open the output
	FILE* fp = fopen(o_path.c_str(), "rb");

	// If output doesn't exist some kind of error occurred
	if (!fp)
	{
		fp = fopen(parse_string("%sacs.err", app_path.c_str()).c_str(), "rt");

		if (!fp)
			wxMessageBox("Compile failed", "Error", wxICON_ERROR);
		else
		{
			// Open a dialog with the contents of acs.err
			fseek(fp, 0, SEEK_END);
			int len = ftell(fp);
			char* text = (char*)malloc(len);
			fseek(fp, 0, SEEK_SET);
			fread(text, len, 1, fp);
			string error = text;
			free(text);

			wxDialog dlg(NULL, -1, "ACC Error Message", wxDefaultPosition, wxDefaultSize);
			wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
			dlg.SetSizer(vbox);
			wxTextCtrl *error_text = new wxTextCtrl(&dlg, -1, "", wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE);
			error_text->SetFont(wxFont(10, wxFONTFAMILY_MODERN, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL));
			error_text->SetValue(string(error.c_str(), error.size()).c_str());
			vbox->Add(error_text, 1, wxEXPAND|wxALL, 4);
			vbox->Add(dlg.CreateButtonSizer(wxOK), 0, wxEXPAND|wxALL, 4);

			dlg.ShowModal();

			fclose(fp);
		}
	}
	else
	{
		fseek(fp, 0, SEEK_END);
		int len = ftell(fp);
		BYTE* data = (BYTE*)malloc(len);
		fseek(fp, 0, SEEK_SET);
		fread(data, len, 1, fp);

		map.behavior->LoadData(data, len);

		fclose(fp);
		remove(o_path.c_str());

		wxMessageBox("Compiled Successfully");
	}
}

void ScriptEditor::text_changed(wxCommandEvent &event)
{
	//text_script->SetStyle(0, 4, wxTextAttr(wxColour(255, 0, 0)));
	//text_script->SetStyle(5, 6, wxTextAttr(wxColour(0, 0, 0)));
}
