
#include "main.h"
#include "tex_browser.h"
#include "textures.h"
#include "map.h"
#include "draw.h"

#include <wx/spinctrl.h>

CVAR(Int, browser_columns, 6, CVAR_SAVE)
CVAR(Int, browser_sort, 1, CVAR_SAVE)

extern wxGLContext *gl_context;
extern Map map;
extern bool mix_tex;

extern vector<Texture*>	textures;
extern vector<Texture*>	flats;
extern vector<Texture*>	sprites;

/*
static int sort_tex_use(const wxString& first, const wxString& second)
{
	int count1;
	int count2;

	for (int a = 0; a < map.n_sides; a++)
	{
		if (map.sides[a]->tex_lower == first.c_str())
			count1++;
		if (map.sides[a]->tex_middle == first.c_str())
			count1++;
		if (map.sides[a]->tex_upper == first.c_str())
			count1++;

		if (map.sides[a]->tex_lower == second.c_str())
			count2++;
		if (map.sides[a]->tex_middle == second.c_str())
			count2++;
		if (map.sides[a]->tex_upper == second.c_str())
			count2++;
	}

	if (mix_tex)
	{
		for (int a = 0; a < map.n_sectors; a++)
		{
			if (map.sectors[a]->f_tex == first.c_str())
				count1++;
			if (map.sectors[a]->c_tex == first.c_str())
				count1++;

			if (map.sectors[a]->f_tex == second.c_str())
				count2++;
			if (map.sectors[a]->c_tex == second.c_str())
				count2++;
		}
	}

	return count1 - count2;
}

static int sort_flats_use(const wxString& first, const wxString& second)
{
	int count1;
	int count2;

	for (int a = 0; a < map.n_sectors; a++)
	{
		if (map.sectors[a]->f_tex == first.c_str())
			count1++;
		if (map.sectors[a]->c_tex == first.c_str())
			count1++;

		if (map.sectors[a]->f_tex == second.c_str())
			count2++;
		if (map.sectors[a]->c_tex == second.c_str())
			count2++;
	}

	if (mix_tex)
	{
		for (int a = 0; a < map.n_sides; a++)
		{
			if (map.sides[a]->tex_lower == first.c_str())
				count1++;
			if (map.sides[a]->tex_middle == first.c_str())
				count1++;
			if (map.sides[a]->tex_upper == first.c_str())
				count1++;

			if (map.sides[a]->tex_lower == second.c_str())
				count2++;
			if (map.sides[a]->tex_middle == second.c_str())
				count2++;
			if (map.sides[a]->tex_upper == second.c_str())
				count2++;
		}
	}

	return count1 - count2;
}
*/

TextureBrowser::TextureBrowser(int type, string tex, bool fullscreen)
:	wxDialog(NULL, -1, "Browse Textures", wxDefaultPosition, wxSize(640, 480), wxCAPTION|wxCLOSE_BOX|wxRESIZE_BORDER|wxMAXIMIZE_BOX|wxSYSTEM_MENU|wxWANTS_CHARS)
{
	wxBoxSizer *m_vbox = new wxBoxSizer(wxVERTICAL);
	SetSizer(m_vbox);

	wxBoxSizer *hbox = new wxBoxSizer(wxHORIZONTAL);
	m_vbox->Add(hbox, 1, wxEXPAND|wxALL, 4);

	vscroll = new wxScrollBar(this, BROWSER_VSCROLL, wxDefaultPosition, wxDefaultSize, wxSB_VERTICAL);
	vscroll->SetScrollbar(0, 1, 10, 1);
	browse_area = new TexBrowseCanvas(this);

	hbox->Add(browse_area, 1, wxEXPAND);
	hbox->Add(vscroll, 0, wxEXPAND);

	wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
	hbox->Add(vbox, 0, wxEXPAND|wxALL, 4);

	search = new wxTextCtrl(this, BROWSER_SEARCH, "");
	vbox->Add(new wxStaticText(this, -1, "Search:"), 0, wxBOTTOM, 2);
	vbox->Add(search, 0, wxEXPAND|wxBOTTOM, 8);

	wxString sort[] = {
		"Alphabetical",
		"Most Used",
	};

	sorting = new wxChoice(this, BROWSER_SORTING, wxDefaultPosition, wxDefaultSize, 2, sort);
	sorting->Select(browser_sort);
	vbox->Add(new wxStaticText(this, -1, "Sorting:"), 0, wxBOTTOM, 2);
	vbox->Add(sorting, 0, wxEXPAND|wxBOTTOM, 8);

	hbox = new wxBoxSizer(wxHORIZONTAL);
	vbox->Add(hbox, 0, wxEXPAND);

	n_columns = new wxSpinCtrl(this, BROWSER_COLS, wxString::Format("%d", (int)browser_columns), wxDefaultPosition, wxSize(56, -1),
								wxSP_ARROW_KEYS, 4, 10, browser_columns);
	hbox->Add(new wxStaticText(this, -1, "Columns:"), 1, wxALIGN_CENTER_VERTICAL|wxRIGHT, 4);
	hbox->Add(n_columns, 0, wxEXPAND);

	m_vbox->Add(CreateButtonSizer(wxOK|wxCANCEL), 0, wxEXPAND|wxALL, 4);

	// Textures
	if (type == TEXTURES_WALLS || type == 0)
	{
		browse_area->tex_names.Add(_T("-"));

		for (int a = 0; a < textures.size(); a++)
			browse_area->tex_names.Add(textures[a]->name.c_str());

		if (mix_tex)
		{
			for (int a = 0; a < flats.size(); a++)
				browse_area->tex_names.Add(flats[a]->name.c_str());
		}
	}

	// Flats
	if (type == TEXTURES_FLATS || type == 0)
	{
		for (int a = 0; a < flats.size(); a++)
			browse_area->tex_names.Add(flats[a]->name.c_str());

		if (mix_tex)
		{
			for (int a = 0; a < textures.size(); a++)
				browse_area->tex_names.Add(textures[a]->name.c_str());
		}
	}

	// Things
	if (type == TEXTURES_SPRITES)
	{
		vector<string> tnames;
		get_ttype_names(&tnames);

		for (int a = 0; a < tnames.size(); a++)
			browse_area->tex_names.Add(tnames[a].c_str());

		browse_area->sprites = true;
	}

	browse_type = type;
	this->sort(browser_sort, type);

	//else if (browser_sort == 1)
	//{
	//	if (type == TEXTURES_WALLS)
	//		browse_area->tex_names.Sort(sort_tex_use);
	//	else if (type == TEXTURES_FLATS)
	//		browse_area->tex_names.Sort(sort_flats_use);
	//}

	browse_area->search = search;
	browse_area->vscroll = vscroll;
	browse_area->selected_tex = tex;
	browse_area->scroll_to_selected();
	browse_area->parent = this;
	browse_area->SetFocus();

	if (fullscreen)
		ShowFullScreen(true);
}

TextureBrowser::~TextureBrowser()
{
}

string TextureBrowser::get_texture()
{
	return browse_area->selected_tex;
}

void TextureBrowser::sort(int sorting, int type)
{
	// Don't sort things (yet)
	if (type == TEXTURES_SPRITES)
		return;

	// Sort alphabetically
	if (sorting == 0)
		browse_area->tex_names.Sort();

	// Sort by use
	else if (sorting == 1)
	{
		table_t used_textures;
		browse_area->tex_names.Sort();

		// Add all textures to table
		for (DWORD t = 0; t < browse_area->tex_names.size(); t++)
			used_textures.add(wx_to_str(browse_area->tex_names[t]), 0);

		// Get texture use frequencies
		if (TEXTURES_WALLS || mix_tex)
		{
			for (DWORD s = 0; s < map.n_sides; s++)
			{
				if (map.sides[s]->tex_lower != "-")
					used_textures.increment(map.sides[s]->tex_lower, 1);

				if (map.sides[s]->tex_upper != "-")
					used_textures.increment(map.sides[s]->tex_upper, 1);

				if (map.sides[s]->tex_middle != "-")
					used_textures.increment(map.sides[s]->tex_middle, 1);
			}
		}

		if (TEXTURES_FLATS || mix_tex)
		{
			for (DWORD s = 0; s < map.n_sectors; s++)
			{
				if (map.sectors[s]->f_tex != "-")
					used_textures.increment(map.sectors[s]->f_tex, 1);

				if (map.sectors[s]->c_tex != "-")
					used_textures.increment(map.sectors[s]->c_tex, 1);
			}
		}

		// Sort textures by frequency
		used_textures.sort_by_values(false);

		// Re-populate browser texture list in frequency order
		browse_area->tex_names.clear();
		for (int t = 0; t < used_textures.n_rows; t++)
			browse_area->tex_names.Add(used_textures.rows[t]->name.c_str());
	}
}

BEGIN_EVENT_TABLE(TextureBrowser, wxDialog)
	EVT_COMMAND_SCROLL(BROWSER_VSCROLL, TextureBrowser::scroll_event)
	EVT_TEXT(BROWSER_SEARCH, TextureBrowser::search_changed)
	EVT_CHOICE(BROWSER_SORTING, TextureBrowser::sorting_changed)
	EVT_SPINCTRL(BROWSER_COLS, TextureBrowser::columns_changed)
END_EVENT_TABLE()

void TextureBrowser::key_down(wxKeyEvent &event)
{
	//wxMessageBox("Key down");
}

void TextureBrowser::scroll_event(wxScrollEvent &event)
{
	browse_area->redraw();
}

void TextureBrowser::search_changed(wxCommandEvent &event)
{
	browse_area->search_str(search->GetValue().Upper().c_str());
	browse_area->SetFocus();
}

void TextureBrowser::sorting_changed(wxCommandEvent &event)
{
	browser_sort = this->sorting->GetSelection();
	this->sort(browser_sort, browse_type);
	browse_area->scroll_to_selected();
	browse_area->redraw();
}

void TextureBrowser::columns_changed(wxSpinEvent &event)
{
	browser_columns = event.GetPosition();
	browse_area->scroll_to_selected();
	browse_area->redraw();
}


TexBrowseCanvas::TexBrowseCanvas(wxWindow *parent)
:	wxGLCanvas(parent, gl_context, -1, wxDefaultPosition, wxDefaultSize, wxWANTS_CHARS)
{
	sprites = false;
}

TexBrowseCanvas::~TexBrowseCanvas()
{
}

void TexBrowseCanvas::redraw()
{
	SetCurrent();

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	int width = GetClientSize().x / browser_columns;
	int top = vscroll->GetThumbPosition();
	int rows = (tex_names.size() / browser_columns) + 1;

	int sel_index = -1;
	int a = 0;
	for (int row = 0; row < rows; row++)
	{
		for (int col = 0; col < browser_columns; col++)
		{
			if (a >= tex_names.size())
				continue;

			rect_t rect(col * width, (row * width) - top, width, width, 0);

			glLineWidth(2.0f);
			if (selected_tex.c_str() == tex_names[a])
			{
				draw_rect(rect, rgba_t(0, 180, 255, 150, 0), true);
				draw_rect(rect, rgba_t(100, 220, 255, 255, 0), false);
				sel_index = a;
			}

			glLineWidth(1.0f);
			rect.resize(-8, -8);

			if (((row + 1) * width) > top && (row * width) < (top + GetClientSize().y))
			{
				if (!sprites)
				{
					if (tex_names[a] != _T("-"))
						draw_texture_scale(rect, wx_to_str(tex_names[a]), 0);

					draw_text(rect.x1() + (width/2) - 8, rect.y2() - 4, rgba_t(255, 255, 255, 255, 0), 1, tex_names[a].c_str());
				}
				else
				{
					thing_type_t* ttype = get_thing_type_from_name(wx_to_str(tex_names[a]));
					draw_texture_scale(rect, ttype->spritename, 3, rgba_t(255, 255, 255, ttype->alpha * 255));
				}
			}

			a++;
		}
	}

	SwapBuffers();
}

void TexBrowseCanvas::scroll_to_selected()
{
	int rows = (tex_names.size() / browser_columns) + 1;
	int width = GetClientSize().x / browser_columns;
	int top = vscroll->GetThumbPosition();
	int bottom = top + GetClientSize().y;

	int a = 0;
	for (int y = 0; y < rows; y++)
	{
		for (int x = 0; x < browser_columns; x++)
		{
			if (tex_names[a] == selected_tex.c_str())
			{
				if (y * width < top)
					vscroll->SetThumbPosition(y * width);

				if ((y+1) * width > bottom)
					vscroll->SetThumbPosition(((y+1) * width) - GetClientSize().y);

				return;
			}
			a++;

			if (a >= tex_names.size())
				return;
		}
	}
}

void TexBrowseCanvas::search_str(string str)
{
	for (int a = 0; a < tex_names.size(); a++)
	{
		if (tex_names[a].size() < str.size())
			continue;

		bool match = true;
		for (int c = 0; c < str.size(); c++)
		{
			if (tex_names[a][c] != str[c])
				match = false;
		}

		if (match)
		{
			selected_tex = tex_names[a];
			scroll_to_selected();
			redraw();
			return;
		}
	}
}

BEGIN_EVENT_TABLE(TexBrowseCanvas, wxGLCanvas)
	EVT_PAINT(TexBrowseCanvas::paint)
	EVT_SIZE(TexBrowseCanvas::resize)
	EVT_KEY_DOWN(TexBrowseCanvas::key_down)
	EVT_MOUSE_EVENTS(TexBrowseCanvas::mouse_event)
END_EVENT_TABLE()

void TexBrowseCanvas::paint(wxPaintEvent &event)
{
	wxPaintDC dc(this);

	redraw();
}

void TexBrowseCanvas::resize(wxSizeEvent &event)
{
	int width = GetClientSize().x / browser_columns;
	int rows = (tex_names.size() / browser_columns) + 1;
	int rows_page = GetClientSize().y / width;
	vscroll->SetScrollbar(vscroll->GetThumbPosition(), GetClientSize().y, rows*width, rows_page*width);
	scroll_to_selected();

	SetCurrent();

	glViewport(0, 0, GetClientSize().x, GetClientSize().y);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(0.0f, GetClientSize().x, GetClientSize().y, 0.0f, -1.0f, 1.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	redraw();
}

void TexBrowseCanvas::key_down(wxKeyEvent &event)
{
	//wxMessageBox("Key down");
	int width = GetClientSize().x / browser_columns;
	int rows = GetClientSize().y / width;

	int index = 0;
	for (int a = 0; a < tex_names.size(); a++)
	{
		if (tex_names[a] == selected_tex.c_str())
			index = a;
	}
	
	bool done = false;

	if (event.GetKeyCode() == WXK_UP)
	{
		done = true;
		if (index - browser_columns >= 0)
			index -= browser_columns;
	}

	if (event.GetKeyCode() == WXK_LEFT)
	{
		done = true;
		if (index > 0)
			index--;
	}

	if (event.GetKeyCode() == WXK_RIGHT)
	{
		done = true;
		if (index < tex_names.size() - 1)
			index++;
	}

	if (event.GetKeyCode() == WXK_DOWN)
	{
		done = true;
		if (index < tex_names.size() - browser_columns)
			index += browser_columns;
	}

	if (event.GetKeyCode() == WXK_PAGEUP ||
		event.GetKeyCode() == WXK_PRIOR)
	{
		done = true;
		if (index - (browser_columns * rows) >= 0)
			index -= (browser_columns * rows);
		else
			index = 0;
	}

	if (event.GetKeyCode() == WXK_PAGEDOWN ||
		event.GetKeyCode() == WXK_NEXT)
	{
		done = true;
		if (index < tex_names.size() - (browser_columns * rows))
			index += (browser_columns * rows);
		else
			index = tex_names.size() - 1;
	}

	if (done)
	{
		selected_tex = tex_names[index];
		scroll_to_selected();
		redraw();
		return;
	}

	search->EmulateKeyPress(event);
}

void TexBrowseCanvas::mouse_event(wxMouseEvent &event)
{
	SetFocus();

	int width = GetClientSize().x / browser_columns;

	// Left click
	if (event.LeftDown())
	{
		int row = (vscroll->GetThumbPosition() + event.GetPosition().y) / width;
		int col = event.GetPosition().x / width;
		int index = (row * browser_columns) + col;

		if (index < tex_names.size())
			selected_tex = tex_names[index];

		redraw();
	}

	// Double left click
	if (event.LeftDClick())
	{
		parent->EndModal(wxID_OK);
		return;
	}

	// Scrolling
	if (event.GetWheelRotation() > 0)
	{
		vscroll->SetThumbPosition(vscroll->GetThumbPosition() - width);
		redraw();
	}
	else if (event.GetWheelRotation() < 0)
	{
		vscroll->SetThumbPosition(vscroll->GetThumbPosition() + width);
		redraw();
	}
}
